@@ -14,16 +14,71 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-"""Functionality for selecting a host from pre-defined list."""
+"""Functionality for selecting a host from pre-defined list.
+
+Ranking/filtering hosts can be achieved using Python expressions which work
+with the `psutil` interfaces.
+
+These expressions are used-defined, buy run a restricted evluation environment
+where only certain whitelisted operations are permitted.
+
+Examples:
+    >>> RankingExpressionEvaluator('1 + 1')
+    2
+    >>> RankingExpressionEvaluator('1 * -1')
+    -1
+    >>> RankingExpressionEvaluator('1 < a', a=2)
+    True
+    >>> RankingExpressionEvaluator('1 in (1, 2, 3)')
+    True
+    >>> import psutil
+    >>> RankingExpressionEvaluator(
+    ...     'a.available > 0',
+    ...     a=psutil.virtual_memory()
+    ... )
+    True
+
+    If you try to get it to do something you're not allowed to:
+    >>> RankingExpressionEvaluator('open("foo")')
+    Traceback (most recent call last):
+    ValueError: Invalid expression: open("foo")
+    "Call" not permitted
+
+    >>> RankingExpressionEvaluator('import sys')
+    Traceback (most recent call last):
+    ValueError: invalid syntax: import sys
+
+    If you try to get hold of something you aren't supposed to:
+    >>> answer = 42  # only variables explicitly passed in should work
+    >>> RankingExpressionEvaluator('answer')
+    Traceback (most recent call last):
+    NameError: name 'answer' is not defined
+
+    If you try to do something which doesn't make sense:
+    >>> RankingExpressionEvaluator('a.b.c')  # no value "a.b.c"
+    Traceback (most recent call last):
+    NameError: name 'a' is not defined
+
+"""
+
 import ast
 from collections import namedtuple
 from functools import lru_cache
 from io import BytesIO
 import json
 import random
+from socket import gaierror
 from time import sleep
 import token
 from tokenize import tokenize
+from typing import (
+    Dict,
+    Iterable,
+    List,
+    Optional,
+    Set,
+    Tuple,
+)
 
 from cylc.flow import LOG
 from cylc.flow.cfgspec.glbl_cfg import glbl_cfg
@@ -32,9 +87,31 @@ from cylc.flow.exceptions import (
     HostSelectException,
     NoHostsError,
 )
-from cylc.flow.hostuserutil import get_fqdn_by_host, is_remote_host
-from cylc.flow.remote import run_cmd, cylc_server_cmd
+from cylc.flow.hostuserutil import (
+    get_fqdn_by_host,
+    is_remote_host,
+)
+from cylc.flow.remote import (
+    cylc_server_cmd,
+    run_cmd,
+)
 from cylc.flow.terminal import parse_dirty_json
+from cylc.flow.util import restricted_evaluator
+
+
+# evaluates ranking expressions
+# (see module docstring for examples)
+RankingExpressionEvaluator = restricted_evaluator(
+    ast.Expression,
+    # variables
+    ast.Name, ast.Load, ast.Attribute, ast.Subscript, ast.Index,
+    # opers
+    ast.BinOp, ast.operator, ast.UnaryOp, ast.unaryop,
+    # types
+    ast.Num, ast.Str,
+    # comparisons
+    ast.Compare, ast.cmpop, ast.List, ast.Tuple,
+)
 
 
 GLBL_CFG_STR = 'global.cylc[scheduler][run hosts]ranking'
@@ -58,14 +135,19 @@ def select_workflow_host(cached=True):
     Raises:
         HostSelectException:
             See `select_host` for details.
-        socket.gaierror:
-            See `select_host` for details.
 
     """
     # get the global config, if cached = False a new config instance will
     # be returned with the up-to-date configuration.
     global_config = glbl_cfg(cached=cached)
 
+    # condemned hosts may be suffixed with an "!" to activate "force mode"
+    blacklist = []
+    for host in global_config.get(['scheduler', 'run hosts', 'condemned'], []):
+        if host.endswith('!'):
+            host = host[:-1]
+        blacklist.append(host)
+
     return select_host(
         # list of workflow hosts
         global_config.get([
@@ -76,19 +158,17 @@ def select_workflow_host(cached=True):
             'scheduler', 'run hosts', 'ranking'
         ]),
         # list of condemned hosts
-        blacklist=global_config.get(
-            ['scheduler', 'run hosts', 'condemned']
-        ),
+        blacklist=blacklist,
         blacklist_name='condemned host'
     )
 
 
 def select_host(
-    hosts,
-    ranking_string=None,
-    blacklist=None,
-    blacklist_name=None
-):
+    hosts: List[str],
+    ranking_string: Optional[str] = None,
+    blacklist: Optional[Iterable[str]] = None,
+    blacklist_name: Optional[str] = None,
+) -> Tuple[str, str]:
     """Select a host from the provided list.
 
     If no ranking is provided (in `ranking_string`) then random selection
@@ -105,21 +185,21 @@ def select_host(
 
                # only consider hosts with less than 70% cpu usage
                # and a server load of less than 5
-               cpu_percent() < 70
+               cpu_percent(1) < 70
                getloadavg()[0] < 5
 
             And or Python statements to rank hosts by e.g::
 
                # rank by used cpu, then by load average as a tie-break
                # (lower scores are better)
-               cpu_percent()
+               cpu_percent(1)
                getloadavg()
 
             Comments are allowed using `#` but not inline comments.
-        blacklist (list):
+        blacklist:
             List of host names to filter out.
             Can be short host names (do not have to be fqdn values)
-        blacklist_name (str):
+        blacklist_name:
             The reason for blacklisting these hosts
             (used for exceptions).
 
@@ -127,39 +207,42 @@ def select_host(
         HostSelectException:
             In the event that no hosts are available / meet the specified
             criterion.
-        socket.gaierror:
-            This may be raised in the event of unknown host names
-            for some installations or not for others.
+            This may also be raised in the event of unknown host names.
 
     Returns:
         tuple - (hostname, fqdn) the chosen host
 
-        hostname (str):
+        hostname:
             The hostname as provided to this function.
-        fqdn (str):
+        fqdn:
             The fully qualified domain name of this host.
 
     """
-    # standardise host names - remove duplicate items
-    hostname_map = {  # note dictionary keys filter out duplicates
-        get_fqdn_by_host(host): host
-        for host in hosts
-    }
-    hosts = list(hostname_map)
-    if blacklist:
-        blacklist = list(set(map(get_fqdn_by_host, blacklist)))
-
     # dict of conditions and whether they have been met (for error reporting)
-    data = {
-        host: {}
-        for host in hosts
-    }
+    data: Dict[str, dict] = {}
+
+    # standardise host names - remove duplicate items
+    hostname_map = {}  # note dictionary keys filter out duplicates
+    for host in hosts:
+        try:
+            hostname_map[get_fqdn_by_host(host)] = host
+        except gaierror as exc:
+            data.setdefault(host, {})[type(exc).__name__] = str(exc)
+    hosts = list(hostname_map)
 
     # filter out `filter_hosts` if provided
     if blacklist:
-        hosts, data = _filter_by_hostname(
+        blacklist_fqdns: Set[str] = set()
+        for host in blacklist:
+            try:
+                blacklist_fqdns.add(get_fqdn_by_host(host))
+            except gaierror as exc:
+                LOG.warning(
+                    f'Could not resolve blacklisted host {host}: {exc}'
+                )
+        hosts = _filter_by_hostname(
             hosts,
-            blacklist,
+            blacklist_fqdns,
             blacklist_name,
             data=data
         )
@@ -172,29 +255,32 @@ def select_host(
     if ranking_string:
         # parse rankings
         rankings = list(_get_rankings(ranking_string))
-        data['ranking'] = ranking_string
 
     if not rankings:
         # no metrics or ranking required, pick host at random
-        hosts = [random.choice(list(hosts))]  # nosec
-
-    if not rankings and len(hosts) == 1:
-        return hostname_map[hosts[0]], hosts[0]
+        random.shuffle(hosts)
+        for host in hosts:
+            if (not is_remote_host(host)) or (
+                # check host is contactable
+                _get_metrics([host], [], data)
+            ):
+                return hostname_map[host], host
+        raise HostSelectException(data)
 
     # filter and sort by rankings
     metrics = list({x for x, _ in rankings})  # required metrics
-    results, data = _get_metrics(  # get data from each host
-        hosts, metrics, data)
+    # get data from each host
+    results = _get_metrics(hosts, metrics, data)
     hosts = list(results)  # some hosts might not be contactable
 
     # stop here if we don't need to proceed
     if not hosts:
         # no hosts provided / left after filtering
-        raise HostSelectException(data)
+        raise HostSelectException(data, ranking_string)
     if not rankings and len(hosts) == 1:
         return hostname_map[hosts[0]], hosts[0]
 
-    hosts, data = _filter_by_ranking(
+    hosts = _filter_by_ranking(
         # filter by rankings, sort by ranking
         hosts,
         rankings,
@@ -204,53 +290,61 @@ def select_host(
 
     if not hosts:
         # no hosts provided / left after filtering
-        raise HostSelectException(data)
+        raise HostSelectException(data, ranking_string)
 
     return hostname_map[hosts[0]], hosts[0]
 
 
 def _filter_by_hostname(
-    hosts,
-    blacklist,
-    blacklist_name=None,
-    data=None
-):
-    """Filter out any hosts present in `blacklist`.
+    hosts: Iterable[str],
+    blacklist: Iterable[str],
+    blacklist_name: Optional[str],
+    data: Dict[str, dict],
+) -> List[str]:
+    """Return hosts, having filtered out any present in `blacklist`.
 
     Args:
-        hosts (list):
+        hosts:
             List of host fqdns.
-        blacklist (list):
+        blacklist:
             List of blacklisted host fqdns.
-        blacklist_name (str):
-            The reason for blacklisting these hosts
-            (used for exceptions).
-        data (dict):
+        data:
             Dict of the form {host: {}}
             (used for exceptions).
+        blacklist_name:
+            The reason for blacklisting these hosts
+            (used for exceptions).
 
     Examples
-        >>> _filter_by_hostname(['a'], [], 'meh')
-        (['a'], {'a': {'blacklisted(meh)': False}})
-        >>> _filter_by_hostname(['a', 'b'], ['a'])
-        (['b'], {'a': {'blacklisted': True}, 'b': {'blacklisted': False}})
+        >>> hosts, data = ['a'], {}
+        >>> _filter_by_hostname(hosts, [], 'meh', data)
+        ['a']
+        >>> data
+        {'a': {'blacklisted(meh)': False}}
+
+        >>> hosts, data = ['a', 'b'], {}
+        >>> _filter_by_hostname(hosts, ['a'], None, data)
+        ['b']
+        >>> data
+        {'a': {'blacklisted': True}, 'b': {'blacklisted': False}}
 
     """
-    if not data:
-        data = {host: {} for host in hosts}
-    for host in list(hosts):
-        key = 'blacklisted'
-        if blacklist_name:
-            key = f'{key}({blacklist_name})'
+    key = 'blacklisted'
+    if blacklist_name:
+        key = f'{key}({blacklist_name})'
+
+    ret = []
+    for host in hosts:
+        data.setdefault(host, {})
         if host in blacklist:
-            hosts.remove(host)
             data[host][key] = True
         else:
+            ret.append(host)
             data[host][key] = False
-    return hosts, data
+    return ret
 
 
-def _filter_by_ranking(hosts, rankings, results, data=None):
+def _filter_by_ranking(hosts, rankings, results, data):
     """Filter and rank by the provided rankings.
 
     Args:
@@ -268,32 +362,42 @@ def _filter_by_ranking(hosts, rankings, results, data=None):
 
     Examples:
         # ranking
+        >>> data = {}
         >>> _filter_by_ranking(
         ...     ['a', 'b'],
         ...     [('X', 'RESULT')],
-        ...     {'a': {'X': 123}, 'b': {'X': 234}}
+        ...     {'a': {'X': 123}, 'b': {'X': 234}},
+        ...     data,
         ... )
-        (['a', 'b'], {'a': {}, 'b': {}})
+        ['a', 'b']
+        >>> data
+        {}
 
         # rankings
+        >>> data = {}
         >>> _filter_by_ranking(
         ...     ['a', 'b'],
         ...     [('X', 'RESULT < 200')],
-        ...     {'a': {'X': 123}, 'b': {'X': 234}}
+        ...     {'a': {'X': 123}, 'b': {'X': 234}},
+        ...     data,
         ... )
-        (['a'], {'a': {'X() < 200': True}, 'b': {'X() < 200': False}})
+        ['a']
+        >>> data
+        {'a': {'X() < 200': True}, 'b': {'X() < 200': False}}
 
         # no matching hosts
+        >>> data = {}
         >>> _filter_by_ranking(
         ...     ['a'],
         ...     [('X', 'RESULT > 1')],
-        ...     {'a': {'X': 0}}
+        ...     {'a': {'X': 0}},
+        ...    data,
         ... )
-        ([], {'a': {'X() > 1': False}})
+        []
+        >>> data
+        {'a': {'X() > 1': False}}
 
     """
-    if not data:
-        data = {host: {} for host in hosts}
     good = []
     for host in hosts:
         host_rankings = {}
@@ -301,17 +405,20 @@ def _filter_by_ranking(hosts, rankings, results, data=None):
         for key, expression in rankings:
             item = _reformat_expr(key, expression)
             try:
-                result = _simple_eval(expression, RESULT=results[host][key])
+                result = RankingExpressionEvaluator(
+                    expression,
+                    RESULT=results[host][key],
+                )
             except Exception as exc:
                 raise GlobalConfigError(
                     'Invalid host ranking expression'
                     f'\n    Expression: {item}'
                     f'\n    Configuration: {GLBL_CFG_STR}'
                     f'\n    Error: {exc}'
-                )
+                ) from None
             if isinstance(result, bool):
                 host_rankings[item] = result
-                data[host][item] = result
+                data.setdefault(host, {})[item] = result
             else:
                 host_rank.append(result)
         if all(host_rankings.values()):
@@ -326,90 +433,8 @@ def _filter_by_ranking(hosts, rankings, results, data=None):
         # no ranking, randomise
         random.shuffle(good)
 
-    return (
-        # list of all hosts which passed rankings (sorted by ranking)
-        [host for _, host in good],
-        # data
-        data
-    )
-
-
-class SimpleVisitor(ast.NodeVisitor):
-    """Abstract syntax tree node visitor for simple safe operations."""
-
-    def visit(self, node):
-        if not isinstance(node, self.whitelist):
-            # permit only whitelisted operations
-            raise ValueError(type(node))
-        return super().visit(node)
-
-    whitelist = (
-        ast.Expression,
-        # variables
-        ast.Name, ast.Load, ast.Attribute, ast.Subscript, ast.Index,
-        # opers
-        ast.BinOp, ast.operator, ast.UnaryOp, ast.unaryop,
-        # types
-        ast.Num, ast.Str,
-        # comparisons
-        ast.Compare, ast.cmpop, ast.List, ast.Tuple,
-    )
-
-
-def _simple_eval(expr, **variables):
-    """Safely evaluates simple python expressions.
-
-    Supports a minimal subset of Python operators:
-    * Binary operations
-    * Simple comparisons
-
-    Supports a minimal subset of Python data types:
-    * Numbers
-    * Strings
-    * Tuples
-    * Lists
-
-    Examples:
-        >>> _simple_eval('1 + 1')
-        2
-        >>> _simple_eval('1 * -1')
-        -1
-        >>> _simple_eval('1 < a', a=2)
-        True
-        >>> _simple_eval('1 in (1, 2, 3)')
-        True
-        >>> import psutil
-        >>> _simple_eval('a.available > 0', a=psutil.virtual_memory())
-        True
-
-        If you try to get it to do something you're not allowed to:
-        >>> _simple_eval('open("foo")')
-        Traceback (most recent call last):
-        ValueError: <class '_ast.Call'>
-        >>> _simple_eval('import sys')
-        Traceback (most recent call last):
-        SyntaxError: ...
-
-        If you try to get hold of something you aren't supposed to:
-        >>> answer = 42  # only variables explicitly passed in should work
-        >>> _simple_eval('answer')
-        Traceback (most recent call last):
-        NameError: name 'a' is not defined
-
-        If you try to do something which doesn't make sense:
-        >>> _simple_eval('a.b.c')  # no value "a.b.c"
-        Traceback (most recent call last):
-        NameError: name 'answer' is not defined
-
-    """
-    node = ast.parse(expr.strip(), mode='eval')
-    SimpleVisitor().visit(node)
-    # acceptable use of eval due to restricted language features
-    return eval(  # nosec
-        compile(node, '<string>', 'eval'),
-        {'__builtins__': {}},
-        variables
-    )
+    # list of all hosts which passed rankings (sorted by ranking)
+    return [host for _, host in good]
 
 
 def _get_rankings(string):
@@ -520,7 +545,7 @@ def _deserialise(metrics, data):
     return data
 
 
-def _get_metrics(hosts, metrics, data=None):
+def _get_metrics(hosts, metrics, data):
     """Retrieve host metrics using SSH if necessary.
 
     Note hosts will not appear in the returned results if:
@@ -537,8 +562,11 @@ def _get_metrics(hosts, metrics, data=None):
 
     Examples:
         Command failure (no such attribute of psutil):
-        >>> _get_metrics(['localhost'], [['elephant']])
-        ({}, {'localhost': {'returncode': 2}})
+        >>> data = {}
+        >>> _get_metrics(['localhost'], [['elephant']], data)
+        {}
+        >>> data
+        {'localhost': {'returncode': 2}}
 
     Returns:
         dict - {host: {(function, arg1, arg2, ...): result}}
@@ -546,8 +574,6 @@ def _get_metrics(hosts, metrics, data=None):
     """
     host_stats = {}
     proc_map = {}
-    if not data:
-        data = {host: {} for host in hosts}
 
     # Start up commands on hosts
     cmd = ['psutil']
@@ -574,19 +600,20 @@ def _get_metrics(hosts, metrics, data=None):
             out, err = (stream.strip() for stream in proc.communicate())
             if proc.wait():
                 # Command failed
-                LOG.warning(
-                    'Error evaluating ranking expression on'
-                    f' {host}: \n{err}'
+                msg = (
+                    'Could not contact' if proc.returncode == 255
+                    else 'Error evaluating ranking expression on'
                 )
+                LOG.warning(f'{msg} {host}:\n{err}')
             else:
                 host_stats[host] = dict(zip(
                     metrics,
                     # convert JSON dicts -> namedtuples
                     _deserialise(metrics, parse_dirty_json(out))
                 ))
-            data[host]['returncode'] = proc.returncode
+            data.setdefault(host, {})['returncode'] = proc.returncode
         sleep(0.01)
-    return host_stats, data
+    return host_stats
 
 
 def _reformat_expr(key, expression):
