@@ -19,32 +19,45 @@ import asyncio
 from queue import Queue
 from textwrap import dedent
 from time import sleep
-from typing import TYPE_CHECKING, Any, Dict, Iterable, List, Optional, Union
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Dict,
+    Iterable,
+    List,
+    Optional,
+    Union,
+)
 
-from graphql.execution import ExecutionResult
-from graphql.execution.executors.asyncio import AsyncioExecutor
 import zmq
 from zmq.auth.thread import ThreadAuthenticator
 
-from cylc.flow import LOG, workflow_files
+from cylc.flow import (
+    LOG,
+    __version__ as CYLC_VERSION,
+    workflow_files,
+)
 from cylc.flow.cfgspec.glbl_cfg import glbl_cfg
-from cylc.flow.network.authorisation import authorise
+from cylc.flow.data_messages_pb2 import PbEntireWorkflow
+from cylc.flow.data_store_mgr import DELTAS_MAP
 from cylc.flow.network.graphql import (
-    CylcGraphQLBackend, IgnoreFieldMiddleware, instantiate_middleware
+    CylcExecutionContext,
+    IgnoreFieldMiddleware,
+    instantiate_middleware,
 )
 from cylc.flow.network.publisher import WorkflowPublisher
 from cylc.flow.network.replier import WorkflowReplier
 from cylc.flow.network.resolvers import Resolvers
 from cylc.flow.network.schema import schema
-from cylc.flow.data_store_mgr import DELTAS_MAP
-from cylc.flow.data_messages_pb2 import PbEntireWorkflow  # type: ignore
+
 
 if TYPE_CHECKING:
+    from cylc.flow.network import ResponseDict
     from cylc.flow.scheduler import Scheduler
 
 
 # maps server methods to the protobuf message (for client/UIS import)
-PB_METHOD_MAP = {
+PB_METHOD_MAP: Dict[str, Any] = {
     'pb_entire_workflow': PbEntireWorkflow,
     'pb_data_elements': DELTAS_MAP
 }
@@ -123,6 +136,10 @@ class WorkflowRuntimeServer:
 
     """
     endpoints: Dict[str, object]
+    curve_auth: ThreadAuthenticator
+    """The ZMQ authenticator."""
+    client_pub_key_dir: str
+    """Client public key directory, used by the ZMQ authenticator."""
 
     OPERATE_SLEEP_INTERVAL = 0.2
     STOP_SLEEP_INTERVAL = 0.2
@@ -136,8 +153,6 @@ class WorkflowRuntimeServer:
         self.publisher = None
         self.loop = None
         self.thread = None
-        self.curve_auth = None
-        self.client_pub_key_dir = None
 
         self.schd: 'Scheduler' = schd
         self.resolvers = Resolvers(
@@ -184,10 +199,7 @@ class WorkflowRuntimeServer:
         self.client_pub_key_dir = client_pub_keyinfo.key_path
 
         # Initial load for the localhost key.
-        self.curve_auth.configure_curve(
-            domain='*',
-            location=(self.client_pub_key_dir)
-        )
+        self.configure_curve()
 
         min_, max_ = glbl_cfg().get(['scheduler', 'run hosts', 'ports'])
         self.replier = WorkflowReplier(self, context=self.zmq_context)
@@ -207,6 +219,11 @@ class WorkflowRuntimeServer:
 
         self.operate()
 
+    def configure_curve(self) -> None:
+        self.curve_auth.configure_curve(
+            domain='*', location=self.client_pub_key_dir
+        )
+
     async def stop(self, reason: Union[BaseException, str]) -> None:
         """Stop the TCP servers, and clean up authentication.
 
@@ -245,7 +262,7 @@ class WorkflowRuntimeServer:
         """Orchestrate the receive, send, publish of messages."""
         # Note: this cannot be an async method because the response part
         # of the listener runs the event loop synchronously
-        # (in graphql AsyncioExecutor)
+        # (in graphql schema.execute_async)
         while True:
             if self.waiting_to_stop:
                 # The self.stop() method is waiting for us to signal that we
@@ -267,7 +284,7 @@ class WorkflowRuntimeServer:
             articles = self.publish_queue.get()
             await self.publisher.publish(*articles)
 
-    def receiver(self, message):
+    def receiver(self, message) -> 'ResponseDict':
         """Process incoming messages and coordinate response.
 
         Wrap incoming messages, dispatch them to exposed methods and/or
@@ -282,29 +299,46 @@ class WorkflowRuntimeServer:
         try:
             method = getattr(self, message['command'])
             args = message['args']
-            args.update({'user': message['user']})
             if 'meta' in message:
                 args['meta'] = message['meta']
-        except KeyError:
+        except KeyError as exc:
             # malformed message
-            return {'error': {
-                'message': 'Request missing required field(s).'}}
+            return {
+                'error': {
+                    'message': (
+                        f"Request missing field {exc} required for "
+                        f"Cylc {CYLC_VERSION}"
+                    )
+                },
+                'cylc_version': CYLC_VERSION,
+            }
         except AttributeError:
             # no exposed method by that name
-            return {'error': {
-                'message': 'No method by the name "%s"' % message['command']}}
+            return {
+                'error': {
+                    'message': (
+                        f"No method by the name '{message['command']}' "
+                        f"at Cylc {CYLC_VERSION}"
+                    )
+                },
+                'cylc_version': CYLC_VERSION,
+            }
 
         # generate response
         try:
-            response = method(**args)
+            data = method(**args)
         except Exception as exc:
             # includes incorrect arguments (TypeError)
-            LOG.exception(exc)  # note the error server side
-            import traceback
-            return {'error': {
-                'message': str(exc), 'traceback': traceback.format_exc()}}
+            LOG.exception(exc)  # log the error server side
+            return {
+                'error': {'message': str(exc)},
+                'cylc_version': CYLC_VERSION,
+            }
 
-        return {'data': response}
+        return {
+            'data': data,
+            'cylc_version': CYLC_VERSION,
+        }
 
     def register_endpoints(self):
         """Register all exposed methods."""
@@ -312,7 +346,6 @@ class WorkflowRuntimeServer:
                           for name, obj in self.__class__.__dict__.items()
                           if hasattr(obj, 'exposed')}
 
-    @authorise()
     @expose
     def api(
         self,
@@ -349,7 +382,6 @@ class WorkflowRuntimeServer:
             return '%s\n%s' % (head, tail)
         return 'No method by name "%s"' % endpoint
 
-    @authorise()
     @expose
     def graphql(
         self,
@@ -357,7 +389,7 @@ class WorkflowRuntimeServer:
         variables: Optional[Dict[str, Any]] = None,
         meta: Optional[Dict[str, Any]] = None
     ):
-        """Return the GraphQL schema execution result.
+        """Return the data field of the GraphQL schema execution result.
 
         Args:
             request_string: GraphQL request passed to Graphene.
@@ -367,45 +399,27 @@ class WorkflowRuntimeServer:
         Returns:
             object: Execution result, or a list with errors.
         """
-        try:
-            executed: ExecutionResult = schema.execute(
+        executed = self.loop.run_until_complete(
+            schema.execute_async(
                 request_string,
                 variable_values=variables,
                 context_value={
                     'resolvers': self.resolvers,
                     'meta': meta or {},
                 },
-                backend=CylcGraphQLBackend(),
                 middleware=list(instantiate_middleware(self.middleware)),
-                executor=AsyncioExecutor(),
-                validate=True,  # validate schema (dev only? default is True)
-                return_promise=False,
+                execution_context_class=CylcExecutionContext,
             )
-        except Exception as exc:
-            return 'ERROR: GraphQL execution error \n%s' % exc
+        )
         if executed.errors:
-            errors: List[Any] = []
             for error in executed.errors:
-                LOG.error(error)
-                if hasattr(error, '__traceback__'):
-                    import traceback
-                    formatted_tb = traceback.format_exception(
-                        type(error), error, error.__traceback__
-                    )
-                    LOG.error("".join(formatted_tb))
-                    errors.append({
-                        'error': {
-                            'message': str(error),
-                            'traceback': formatted_tb
-                        }
-                    })
-                    continue
-                errors.append(getattr(error, 'message', None))
-            return errors
+                LOG.warning(f"GraphQL: {error}")
+            # If there are execution errors, it means there was an unexpected
+            # error, so fail the command.
+            raise Exception(*(error.message for error in executed.errors))
         return executed.data
 
     # UIServer Data Commands
-    @authorise()
     @expose
     def pb_entire_workflow(self, **_kwargs) -> bytes:
         """Send the entire data-store in a single Protobuf message.
@@ -416,7 +430,6 @@ class WorkflowRuntimeServer:
         pb_msg = self.schd.data_store_mgr.get_entire_workflow()
         return pb_msg.SerializeToString()
 
-    @authorise()
     @expose
     def pb_data_elements(self, element_type: str, **_kwargs) -> bytes:
         """Send the specified data elements in delta form.
