@@ -35,6 +35,10 @@ if TYPE_CHECKING:
     from optparse import Values
 
 
+class DefaultList(list):
+    """List subclass to indicate unassigned list values in expanded config."""
+
+
 class ParsecConfig:
     """Object wrapper for parsec functions."""
 
@@ -112,7 +116,7 @@ class ParsecConfig:
                     else:
                         if node.default == ConfigNode.UNSET:
                             if node.vdr and node.vdr.endswith('_LIST'):
-                                defs[node.name] = []
+                                defs[node.name] = DefaultList()
                             else:
                                 defs[node.name] = None
                         else:
@@ -145,13 +149,17 @@ class ParsecConfig:
                     cfg = cfg[key]
                 except (KeyError, TypeError):
                     if (
+                        # __MANY__ setting not present:
                         parents in self.manyparents or
+                        # setting not present in __MANY__ section:
                         key in self.spec.get(*parents)
                     ):
-                        raise ItemNotFoundError(itemstr(parents, key))
+                        raise ItemNotFoundError(
+                            itemstr(parents, key)
+                        ) from None
                     raise InvalidConfigError(
                         itemstr(parents, key), self.spec.name
-                    )
+                    ) from None
                 else:
                     parents.append(key)
 
@@ -238,6 +246,13 @@ class ConfigNode(ContextNode):
         options:
             List of possible options.
             TODO: allow this to be a dict with help info
+        depr_options:
+            List of deprecated options. These are not displayed in the docs
+            but are used for backwards compatibility.
+        warn_options:
+            If True, parsec will warn if invalid options are present rather
+            than raising an exception. Any invalid options will be stripped
+            from the config.
         default:
             The default value.
         desc:
@@ -265,7 +280,14 @@ class ConfigNode(ContextNode):
     UNSET = '*value unset*'
 
     __slots__ = ContextNode.__slots__ + (
-        'vdr', 'options', 'default', 'desc', 'display_name', 'meta'
+        'vdr',
+        'options',
+        'depr_options',
+        'warn_options',
+        'default',
+        'desc',
+        'display_name',
+        'meta',
     )
 
     def __init__(
@@ -275,7 +297,9 @@ class ConfigNode(ContextNode):
         default: object = UNSET,
         options: Optional[list] = None,
         desc: Optional[str] = None,
-        meta: Optional['ConfigNode'] = None
+        meta: Optional['ConfigNode'] = None,
+        depr_options: Optional[list] = None,
+        warn_options: bool = False,
     ):
         display_name = name
         if name.startswith('<'):
@@ -298,7 +322,9 @@ class ConfigNode(ContextNode):
         self.display_name = display_name
         self.vdr = vdr
         self.default = default
-        self.options = options
+        self.options = options or []
+        self.depr_options = depr_options or []
+        self.warn_options = warn_options
         self.desc = dedent(desc).strip() if desc else None
         self.meta = meta
 
