@@ -43,6 +43,11 @@ class upgrader:
     SITE_CONFIG = 'site config'
     USER_CONFIG = 'user config'
 
+    DEPR_MSG = (
+        "Deprecated config items were automatically upgraded. "
+        "Please alter your workflow to use the new syntax."
+    )
+
     def __init__(self, cfg, descr):
         """Store the config dict to be upgraded if necessary."""
         self.cfg = cfg
@@ -110,7 +115,7 @@ class upgrader:
                 raise UpgradeError(
                     f'{self.show_keys(keys[:-1], True)}'
                     f' ("{keys[-2]}" should be a [section] not a setting)'
-                )
+                ) from None
         return item
 
     def put_item(self, keys, val):
@@ -194,6 +199,8 @@ class upgrader:
 
     def upgrade(self):
         warnings = OrderedDict()
+        deprecations = False
+        obsoletions = False
         for vn, upgs in self.upgrades.items():
             for u in upgs:
                 try:
@@ -212,6 +219,9 @@ class upgrader:
                         if upg['new']:
                             msg += ' -> ' + self.show_keys(upg['new'],
                                                            upg['is_section'])
+                            deprecations = True
+                        else:
+                            obsoletions = True
                         msg += " - " + upg['cvt'].describe().format(
                             old=old,
                             new=upg['cvt'].convert(old)
@@ -236,7 +246,6 @@ class upgrader:
                             self.put_item(upg['new'],
                                           upg['cvt'].convert(old))
         if warnings:
-            level = WARNING
             if self.descr == self.SITE_CONFIG:
                 # Site level configuration, user cannot easily fix.
                 # Only log at debug level.
@@ -245,9 +254,15 @@ class upgrader:
                 # User level configuration, user should be able to fix.
                 # Log at warning level.
                 level = WARNING
-            LOG.log(level,
-                    'deprecated items were automatically upgraded in '
-                    f'"{self.descr}"')
+            if obsoletions:
+                LOG.log(
+                    level,
+                    "Obsolete config items were automatically deleted."
+                    " Please check your workflow and remove them permanently."
+                )
+            if deprecations:
+                LOG.log(level, self.DEPR_MSG)
+
             for vn, msgs in warnings.items():
                 for msg in msgs:
                     LOG.log(level, ' * (%s) %s', vn, msg)
