@@ -31,6 +31,7 @@ from cylc.flow.exceptions import (
     PlatformLookupError, CylcError, NoHostsError, NoPlatformsError)
 from cylc.flow.cfgspec.glbl_cfg import glbl_cfg
 from cylc.flow.hostuserutil import is_remote_host
+from cylc.flow.run_modes import JOBLESS_MODES
 
 if TYPE_CHECKING:
     from cylc.flow.parsec.OrderedDict import OrderedDictWithDefaults
@@ -56,7 +57,7 @@ SINGLE_HOST_JOB_RUNNERS = ['background', 'at']
 
 # Regex to check whether a string is a command
 HOST_REC_COMMAND = re.compile(r'(`|\$\()\s*(.*)\s*([`)])$')
-PLATFORM_REC_COMMAND = re.compile(r'(\$\()\s*(.*)\s*([)])$')
+PLATFORM_REC_COMMAND = re.compile(r'(\$\()\s*(.*)\s*(\))')
 
 HOST_SELECTION_METHODS = {
     'definition order': lambda goodhosts: goodhosts[0],
@@ -82,7 +83,8 @@ def log_platform_event(
 def get_platform(
     task_conf: Optional[str] = None,
     task_name: str = UNKNOWN_TASK,
-    bad_hosts: Optional[Set[str]] = None
+    bad_hosts: Optional[Set[str]] = None,
+    evaluated_host: Optional[str] = None,
 ) -> Dict[str, Any]:
     ...
 
@@ -91,7 +93,8 @@ def get_platform(
 def get_platform(
     task_conf: Union[dict, 'OrderedDictWithDefaults'],
     task_name: str = UNKNOWN_TASK,
-    bad_hosts: Optional[Set[str]] = None
+    bad_hosts: Optional[Set[str]] = None,
+    evaluated_host: Optional[str] = None,
 ) -> Optional[Dict[str, Any]]:
     ...
 
@@ -107,7 +110,8 @@ def get_platform(
 def get_platform(
     task_conf: Union[str, dict, 'OrderedDictWithDefaults', None] = None,
     task_name: str = UNKNOWN_TASK,
-    bad_hosts: Optional[Set[str]] = None
+    bad_hosts: Optional[Set[str]] = None,
+    evaluated_host: Optional[str] = None,
 ) -> Optional[Dict[str, Any]]:
     """Get a platform.
 
@@ -120,11 +124,13 @@ def get_platform(
         task_name: Help produce more helpful error messages.
         bad_hosts: A set of hosts known to be unreachable (had an ssh 255
             error)
+        evaluated_host: Host name evaluated from platform subshell.
 
     Returns:
         platform: A platform definition dictionary. Uses either
             get_platform() or platform_name_from_job_info(), but to the
-            user these look the same.
+            user these look the same. This will be None if the platform
+            definition uses a subshell.
 
     Raises:
         NoPlatformsError:
@@ -141,7 +147,7 @@ def get_platform(
 
     # NOTE: Do NOT use .get() on OrderedDictWithDefaults -
     # https://github.com/cylc/cylc-flow/pull/4975
-    elif 'platform' in task_conf and task_conf['platform']:
+    if 'platform' in task_conf and task_conf['platform']:
         # Check whether task has conflicting Cylc7 items.
         fail_if_platform_and_host_conflict(task_conf, task_name)
 
@@ -154,24 +160,23 @@ def get_platform(
         # If platform name exists and doesn't clash with Cylc7 Config items.
         return platform_from_name(task_conf['platform'], bad_hosts=bad_hosts)
 
-    else:
-        if get_platform_deprecated_settings(task_conf) == []:
-            # No deprecated items; platform is localhost
-            return platform_from_name()
-        else:
-            # Need to calculate platform
-            # NOTE: Do NOT use .get() on OrderedDictWithDefaults - see above
-            task_job_section = task_conf['job'] if 'job' in task_conf else {}
-            task_remote_section = (
-                task_conf['remote'] if 'remote' in task_conf else {})
-            return platform_from_name(
-                platform_name_from_job_info(
-                    glbl_cfg().get(['platforms']),
-                    task_job_section,
-                    task_remote_section
-                ),
-                bad_hosts=bad_hosts
-            )
+    if get_platform_deprecated_settings(task_conf) == []:
+        # No deprecated items; platform is localhost
+        return platform_from_name()
+
+    # Need to calculate platform
+    # NOTE: Do NOT use .get() on OrderedDictWithDefaults - see above
+    task_job_section = task_conf['job'] if 'job' in task_conf else {}
+    task_remote_section = task_conf['remote'] if 'remote' in task_conf else {}
+    return platform_from_name(
+        platform_name_from_job_info(
+            glbl_cfg().get(['platforms']),
+            task_job_section,
+            task_remote_section,
+            evaluated_host,
+        ),
+        bad_hosts=bad_hosts,
+    )
 
 
 def platform_from_name(
@@ -265,6 +270,12 @@ def platform_from_name(
             platform_data['name'] = platform_name
             return platform_data
 
+    # If platform name in run mode and not otherwise defined:
+    if platform_name in JOBLESS_MODES:
+        platform_data = deepcopy(platforms['localhost'])
+        platform_data['name'] = 'localhost'
+        return platform_data
+
     raise PlatformLookupError(
         f"No matching platform \"{platform_name}\" found")
 
@@ -294,17 +305,19 @@ def get_platform_from_group(
     if bad_hosts:
         platform_names = [
             platform for platform in group['platforms']
-            if any(
-                host not in bad_hosts
-                for host in platform_from_name(platform)['hosts']
-            )
+            if not bad_hosts.issuperset(platform_from_name(platform)['hosts'])
         ]
     else:
         platform_names = group['platforms']
 
-    # Return False if there are no platforms available to be selected.
+    # If there are no platforms available to be selected:
     if not platform_names:
-        raise NoPlatformsError(group_name)
+        hosts_consumed = {
+            host
+            for platform in group['platforms']
+            for host in platform_from_name(platform)['hosts']}
+        raise NoPlatformsError(
+            group_name, hosts_consumed)
 
     # Get the selection method
     method = group['selection']['method']
@@ -319,7 +332,8 @@ def get_platform_from_group(
 def platform_name_from_job_info(
     platforms: Union[dict, 'OrderedDictWithDefaults'],
     job: Dict[str, Any],
-    remote: Dict[str, Any]
+    remote: Dict[str, Any],
+    evaluated_host: Optional[str] = None,
 ) -> str:
     """
     Find out which job platform to use given a list of possible platforms
@@ -374,6 +388,7 @@ def platform_name_from_job_info(
         job: Workflow config [runtime][TASK][job] section.
         remote: Workflow config [runtime][TASK][remote] section.
         platforms: Dictionary containing platform definitions.
+        evaluated_host: Host is the result of evaluating a subshell.
 
     Returns:
         platform: string representing a platform from the global config.
@@ -411,7 +426,9 @@ def platform_name_from_job_info(
 
     # NOTE: Do NOT use .get() on OrderedDictWithDefaults -
     # https://github.com/cylc/cylc-flow/pull/4975
-    if 'host' in remote and remote['host']:
+    if evaluated_host:
+        task_host = evaluated_host
+    elif 'host' in remote and remote['host']:
         task_host = remote['host']
     else:
         task_host = 'localhost'
@@ -651,6 +668,7 @@ def get_install_target_to_platforms_map(
         else:
             install_target = get_install_target_from_platform(platform)
             ret.setdefault(install_target, []).append(platform)
+
     return ret
 
 
