@@ -15,22 +15,49 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """Provide data access object for the workflow runtime database."""
 
+from collections import defaultdict
 from contextlib import suppress
 from dataclasses import dataclass
 from os.path import expandvars
 from pprint import pformat
 import sqlite3
 import traceback
-from typing import TYPE_CHECKING, Iterable, List, Optional, Tuple, Union
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    DefaultDict,
+    Dict,
+    Iterable,
+    List,
+    Optional,
+    Set,
+    Tuple,
+    Union,
+    cast,
+)
 
 from cylc.flow import LOG
 from cylc.flow.exceptions import PlatformLookupError
-from cylc.flow.util import deserialise
 import cylc.flow.flags
+from cylc.flow.flow_mgr import stringify_flow_nums
+from cylc.flow.util import (
+    deserialise_set,
+    serialise_set,
+)
+
 
 if TYPE_CHECKING:
     from pathlib import Path
 
+    from cylc.flow.flow_mgr import FlowNums
+
+
+DbArgDict = Dict[str, Any]
+DbUpdateTuple = Union[
+    Tuple[DbArgDict, DbArgDict],
+    Tuple[str, list]
+]
+
 
 @dataclass
 class CylcWorkflowDAOTableColumn:
@@ -54,7 +81,7 @@ class CylcWorkflowDAOTable:
 
     def __init__(self, name, column_items):
         self.name = name
-        self.columns = []
+        self.columns: List[CylcWorkflowDAOTableColumn] = []
         for column_item in column_items:
             name = column_item[0]
             attrs = {}
@@ -66,7 +93,7 @@ class CylcWorkflowDAOTable:
                 attrs.get("is_primary_key", False)))
         self.delete_queues = {}
         self.insert_queue = []
-        self.update_queues = {}
+        self.update_queues: DefaultDict[str, list] = defaultdict(list)
 
     def get_create_stmt(self):
         """Return an SQL statement to create this table."""
@@ -135,14 +162,23 @@ class CylcWorkflowDAOTable:
                 args.get(column.name, None) for column in self.columns]
         self.insert_queue.append(stmt_args)
 
-    def add_update_item(self, set_args, where_args):
+    def add_update_item(self, item: DbUpdateTuple) -> None:
         """Queue an UPDATE item.
 
+        If stmt is not a string, it should be a tuple (set_args, where_args) -
         set_args should be a dict, with column keys and values to be set.
         where_args should be a dict, update will only apply to rows matching
         all these items.
 
         """
+        if isinstance(item[0], str):
+            stmt = item[0]
+            params = cast('list', item[1])
+            self.update_queues[stmt].extend(params)
+            return
+
+        set_args = item[0]
+        where_args = cast('DbArgDict', item[1])
         set_strs = []
         stmt_args = []
         for column in self.columns:
@@ -162,9 +198,8 @@ class CylcWorkflowDAOTable:
         stmt = self.FMT_UPDATE % {
             "name": self.name,
             "set_str": set_str,
-            "where_str": where_str}
-        if stmt not in self.update_queues:
-            self.update_queues[stmt] = []
+            "where_str": where_str
+        }
         self.update_queues[stmt].append(stmt_args)
 
 
@@ -251,6 +286,7 @@ class CylcWorkflowDAO:
             ["flow_nums"],
             ["is_manual_submit", {"datatype": "INTEGER"}],
             ["try_num", {"datatype": "INTEGER"}],
+            # This is used to store simulation task start time across restarts.
             ["time_submit"],
             ["time_submit_exit"],
             ["submit_status", {"datatype": "INTEGER"}],
@@ -298,6 +334,8 @@ class CylcWorkflowDAO:
             ["prereq_output", {"is_primary_key": True}],
             ["satisfied"],
         ],
+        # The xtriggers table holds the function signature and result of
+        # already-satisfied (the scheduler no longer needs to call them).
         TABLE_XTRIGGERS: [
             ["signature", {"is_primary_key": True}],
             ["results"],
@@ -389,15 +427,18 @@ class CylcWorkflowDAO:
         """
         self.tables[table_name].add_insert_item(args)
 
-    def add_update_item(self, table_name, set_args, where_args=None):
+    def add_update_item(
+        self, table_name: str, item: DbUpdateTuple
+    ) -> None:
         """Queue an UPDATE item for a given table.
 
+        If stmt is not a string, it should be a tuple (set_args, where_args) -
         set_args should be a dict, with column keys and values to be set.
         where_args should be a dict, update will only apply to rows matching
         all these items.
 
         """
-        self.tables[table_name].add_update_item(set_args, where_args)
+        self.tables[table_name].add_update_item(item)
 
     def close(self) -> None:
         """Explicitly close the connection."""
@@ -463,20 +504,49 @@ class CylcWorkflowDAO:
 
         # something went wrong
         # (includes DB file not found, transaction processing issue, db locked)
-        except sqlite3.Error:
+        except sqlite3.Error as e:
+            # Detailed error codes are only available for python >= 3.11
+            if hasattr(e, "sqlite_errorcode"):
+                error_code = str(e.sqlite_errorcode)
+            else:
+                error_code = "Not available"
+
+            if hasattr(e, "sqlite_errorname"):
+                error_name = e.sqlite_errorname
+            else:
+                error_name = "Not available"
+
             if not self.is_public:
                 # incase this isn't a filesystem issue, log the statements
                 # which make up the transaction to assist debug
                 LOG.error(
-                    'An error occurred when writing to the database,'
+                    'An error occurred when writing to the database %(file)s,'
                     ' this is probably a filesystem issue.'
-                    f' The attempted transaction was:\n{pformat(sql_queue)}'
+                    ' The error was: %(error)s'
+                    ' SQLite error code: %(error_code)s'
+                    ' SQLite error name: %(error_name)s'
+                    ' The attempted transaction was:\n %(transaction)s' % {
+                        "file": self.db_file_name,
+                        "error": str(e),
+                        "error_code": error_code,
+                        "error_name": error_name,
+                        "transaction": pformat(sql_queue)
+                    }
                 )
                 raise
             self.n_tries += 1
-            LOG.warning(
-                "%(file)s: write attempt (%(attempt)d) did not complete\n" % {
-                    "file": self.db_file_name, "attempt": self.n_tries})
+            LOG.info(
+                "%(file)s: write attempt (%(attempt)d)"
+                " did not complete: %(error)s\n"
+                " SQLite error code: %(error_code)s\n"
+                " SQLite error name: %(error_name)s" % {
+                    "file": self.db_file_name,
+                    "attempt": self.n_tries,
+                    "error": str(e),
+                    "error_code": error_code,
+                    "error_name": error_name
+                }
+            )
             if self.conn is not None:
                 with suppress(sqlite3.Error):
                     self.conn.rollback()
@@ -490,7 +560,7 @@ class CylcWorkflowDAO:
                 table.update_queues.clear()
             # Report public database retry recovery if necessary
             if self.n_tries:
-                LOG.warning(
+                LOG.info(
                     "%(file)s: recovered after (%(attempt)d) attempt(s)\n" % {
                         "file": self.db_file_name, "attempt": self.n_tries})
             self.n_tries = 0
@@ -519,19 +589,22 @@ class CylcWorkflowDAO:
         try:
             self.connect()
             self.conn.executemany(stmt, stmt_args_list)
-        except sqlite3.Error:
+        except sqlite3.Error as e:
             if not self.is_public:
                 raise
             if cylc.flow.flags.verbosity > 1:
                 traceback.print_exc()
             err_log = (
                 "cannot execute database statement:\n"
-                "file=%(file)s:\nstmt=%(stmt)s"
-            ) % {"file": self.db_file_name, "stmt": stmt}
+                "file=%(file)s:\nstmt=%(stmt)s\nerror=%(error)s"
+            ) % {"file": self.db_file_name, "stmt": stmt, "error": str(e)}
             for i, stmt_args in enumerate(stmt_args_list):
                 err_log += ("\nstmt_args[%(i)d]=%(stmt_args)s" % {
                     "i": i, "stmt_args": stmt_args})
-            LOG.warning(err_log)
+            if self.is_public:
+                LOG.info(err_log)
+            else:
+                LOG.warning(err_log)
             raise
 
     def pre_select_broadcast_states(self, order=None):
@@ -562,10 +635,10 @@ class CylcWorkflowDAO:
                 key, value
             FROM
                 {self.TABLE_WORKFLOW_PARAMS}
-        '''  # nosec (table name is code constant)
+        '''  # nosec B608 (table name is code constant)
         return self.connect().execute(stmt)
 
-    def select_workflow_flows(self, flow_nums):
+    def select_workflow_flows(self, flow_nums: Iterable[int]):
         """Return flow data for selected flows."""
         stmt = rf'''
             SELECT
@@ -573,8 +646,8 @@ class CylcWorkflowDAO:
             FROM
                 {self.TABLE_WORKFLOW_FLOWS}
             WHERE
-                flow_num in ({','.join(str(f) for f in flow_nums)})
-        '''  # nosec (table name is code constant, flow_nums just integers)
+                flow_num in ({stringify_flow_nums(flow_nums)})
+        '''  # nosec B608 (table name is code constant, flow_nums just ints)
         flows = {}
         for flow_num, start_time, descr in self.connect().execute(stmt):
             flows[flow_num] = {
@@ -590,7 +663,7 @@ class CylcWorkflowDAO:
                 MAX(flow_num)
             FROM
                 {self.TABLE_WORKFLOW_FLOWS}
-        '''  # nosec (table name is code constant)
+        '''  # nosec B608 (table name is code constant)
         return self.connect().execute(stmt).fetchone()[0]
 
     def select_workflow_params_restart_count(self):
@@ -602,7 +675,7 @@ class CylcWorkflowDAO:
                 {self.TABLE_WORKFLOW_PARAMS}
             WHERE
                 key == 'n_restart'
-        """  # nosec (table name is code constant)
+        """  # nosec B608 (table name is code constant)
         result = self.connect().execute(stmt).fetchone()
         return int(result[0]) if result else 0
 
@@ -618,7 +691,7 @@ class CylcWorkflowDAO:
                         key, value
                     FROM
                         {self.TABLE_WORKFLOW_TEMPLATE_VARS}
-                '''  # nosec (table name is code constant)
+                '''  # nosec B608 (table name is code constant)
         )):
             callback(row_idx, list(row))
 
@@ -635,17 +708,18 @@ class CylcWorkflowDAO:
                 {",".join(attrs)}
             FROM
                 {self.TABLE_TASK_ACTION_TIMERS}
-        '''  # nosec
+        '''  # nosec B608
         # * table name is code constant
         # * attrs are code constants
         for row_idx, row in enumerate(self.connect().execute(stmt)):
             callback(row_idx, list(row))
 
-    def select_task_job(self, cycle, name, submit_num=None):
+    def select_task_job(
+        self, cycle: str, name: str, submit_num: str | int | None = None
+    ) -> dict[str, Any] | None:
         """Select items from task_jobs by (cycle, name, submit_num).
 
         :return: a dict for mapping keys to the column values
-        :rtype: dict
         """
         keys = []
         for column in self.tables[self.TABLE_TASK_JOBS].columns[3:]:
@@ -661,10 +735,10 @@ class CylcWorkflowDAO:
                     AND name==?
                 ORDER BY
                     submit_num DESC LIMIT 1
-            '''  # nosec
+            '''  # nosec B608
             # * table name is code constant
             # * keys are code constants
-            stmt_args = [cycle, name]
+            stmt_args: list[Any] = [cycle, name]
         else:
             stmt = rf'''
                 SELECT
@@ -675,18 +749,17 @@ class CylcWorkflowDAO:
                     cycle==?
                     AND name==?
                     AND submit_num==?
-            '''  # nosec
+            '''  # nosec B608
             # * table name is code constant
             # * keys are code constants
             stmt_args = [cycle, name, submit_num]
-        try:
+        with suppress(sqlite3.DatabaseError):
             for row in self.connect().execute(stmt, stmt_args):
                 ret = {}
                 for key, value in zip(keys, row):
                     ret[key] = value
                 return ret
-        except sqlite3.DatabaseError:
-            return None
+        return None
 
     def select_jobs_for_restart(self, callback):
         """Select from task_pool+task_states+task_jobs for restart.
@@ -758,46 +831,58 @@ class CylcWorkflowDAO:
                 platform_name
             FROM
                 {self.TABLE_TASK_JOBS}
-        '''  # nosec (table name is code constant)
+        '''  # nosec B608 (table name is code constant)
         return {i[0] for i in self.connect().execute(stmt)}
 
-    def select_submit_nums(self, name, point):
-        """Select submit_num and flow_nums from task_states table.
-
-        Fetch submit number and flow_nums for spawning tasks.
-
-        Return: {submit_num: (flow_wait, flow_nums)}
-
-        Args:
-            name: task name
-            point: task cycle point (str)
+    def select_prev_instances(
+        self, name: str, point: str
+    ) -> List[Tuple[int, bool, Set[int], str]]:
+        """Select task_states table info about previous instances of a task.
 
+        Flow merge results in multiple entries for the same submit number.
         """
         # Ignore bandit false positive: B608: hardcoded_sql_expressions
         # Not an injection, simply putting the table name in the SQL query
         # expression as a string constant local to this module.
-        stmt = (  # nosec
-            r"SELECT flow_nums,submit_num,flow_wait FROM %(name)s"
+        stmt = (  # nosec B608
+            r"SELECT flow_nums,submit_num,flow_wait,status FROM %(name)s"
             r" WHERE name==? AND cycle==?"
         ) % {"name": self.TABLE_TASK_STATES}
-        ret = {}
-        for flow_nums_str, submit_num, flow_wait in self.connect().execute(
-                stmt, (name, point,)):
-            ret[submit_num] = (flow_wait == 1, deserialise(flow_nums_str))
-        return ret
+        return [
+            (
+                submit_num,
+                flow_wait == 1,
+                deserialise_set(flow_nums_str),
+                status
+            )
+            for flow_nums_str, submit_num, flow_wait, status in (
+                self.connect().execute(stmt, (name, point,))
+            )
+        ]
 
-    def select_latest_flow_nums(self):
+    def select_latest_flow_nums(self) -> Optional['FlowNums']:
         """Return a list of the most recent previous flow numbers."""
         stmt = rf'''
-            SELECT flow_nums, MAX(time_created) FROM {self.TABLE_TASK_STATES}
-        '''  # nosec (table name is code constant)
-        flow_nums_str = list(self.connect().execute(stmt))[0][0]
-        return deserialise(flow_nums_str)
+            SELECT
+                flow_nums, MAX(time_created)
+            FROM
+                {self.TABLE_TASK_STATES}
+            WHERE
+                flow_nums != ?
+        '''  # nosec B608 (table name is code constant)
+        # Exclude flow=none:
+        params = [serialise_set()]
+        flow_nums_str = self.connect().execute(stmt, params).fetchone()[0]
+        if flow_nums_str:
+            return deserialise_set(flow_nums_str)
+        return None
 
-    def select_task_outputs(self, name, point):
+    def select_task_outputs(
+        self, name: str, point: str
+    ) -> 'Dict[str, FlowNums]':
         """Select task outputs for each flow.
 
-        Return: {outputs_list: flow_nums_set}
+        Return: {outputs_dict_str: flow_nums_set}
 
         """
         stmt = rf'''
@@ -807,11 +892,13 @@ class CylcWorkflowDAO:
                {self.TABLE_TASK_OUTPUTS}
             WHERE
                 name==? AND cycle==?
-        '''  # nosec (table name is code constant)
-        ret = {}
-        for flow_nums, outputs in self.connect().execute(stmt, (name, point,)):
-            ret[outputs] = deserialise(flow_nums)
-        return ret
+        '''  # nosec B608 (table name is code constant)
+        return {
+            outputs: deserialise_set(flow_nums)
+            for flow_nums, outputs in self.connect().execute(
+                stmt, (name, point,)
+            )
+        }
 
     def select_xtriggers_for_restart(self, callback):
         stmt = rf'''
@@ -819,7 +906,7 @@ class CylcWorkflowDAO:
                 signature, results
             FROM
                 {self.TABLE_XTRIGGERS}
-        '''  # nosec (table name is code constant)
+        '''  # nosec B608 (table name is code constant)
         for row_idx, row in enumerate(self.connect().execute(stmt, [])):
             callback(row_idx, list(row))
 
@@ -829,7 +916,7 @@ class CylcWorkflowDAO:
                 cycle, name, output
             FROM
                 {self.TABLE_ABS_OUTPUTS}
-        '''  # nosec (table name is code constant)
+        '''  # nosec B608 (table name is code constant)
         for row_idx, row in enumerate(self.connect().execute(stmt, [])):
             callback(row_idx, list(row))
 
@@ -942,7 +1029,7 @@ class CylcWorkflowDAO:
                 cycle == ? AND
                 name == ? AND
                 flow_nums == ?
-        """  # nosec (table name is code constant)
+        """  # nosec B608 (table name is code constant)
         stmt_args = [cycle, name, flow_nums]
         return list(self.connect().execute(stmt, stmt_args))
 
@@ -953,7 +1040,7 @@ class CylcWorkflowDAO:
                 name, cycle
             FROM
                 {self.TABLE_TASKS_TO_HOLD}
-        '''  # nosec (table name is code constant)
+        '''  # nosec B608 (table name is code constant)
         return list(self.connect().execute(stmt))
 
     def select_task_times(self):
@@ -975,7 +1062,7 @@ class CylcWorkflowDAO:
                 {self.TABLE_TASK_JOBS}
             WHERE
                 run_status = 0
-        """  # nosec (table name is code constant)
+        """  # nosec B608 (table name is code constant)
         columns = (
             'name', 'cycle', 'host', 'job_runner',
             'submit_time', 'start_time', 'succeed_time'
