@@ -64,6 +64,9 @@ from optparse import SUPPRESS_HELP
 import sys
 from typing import TYPE_CHECKING, Iterable, List, Tuple
 
+from metomi.isodatetime.exceptions import ISO8601SyntaxError
+from metomi.isodatetime.parsers import DurationParser
+
 from cylc.flow import LOG
 from cylc.flow.clean import init_clean, get_contained_workflows
 from cylc.flow.exceptions import CylcError, InputError
@@ -120,9 +123,11 @@ def get_option_parser():
 
     parser.add_option(
         '--timeout',
-        help=("The number of seconds to wait for cleaning to take place on "
-              r"remote hosts before cancelling. Default: %default."),
-        action='store', default='120', dest='remote_timeout'
+        help=(
+            "The length of time to wait for cleaning to take place on "
+            r"remote hosts before cancelling. Default: %default."
+        ),
+        action='store', default='PT5M', dest='remote_timeout'
     )
 
     parser.add_option(
@@ -138,6 +143,24 @@ def get_option_parser():
 CleanOptions = Options(get_option_parser())
 
 
+def parse_timeout(opts: 'Values') -> None:
+    """Parse timeout as ISO 8601 duration or number of seconds."""
+    if opts.remote_timeout:
+        try:
+            timeout = int(
+                DurationParser().parse(opts.remote_timeout).get_seconds()
+            )
+        except ISO8601SyntaxError:
+            try:
+                timeout = int(opts.remote_timeout)
+            except ValueError:
+                raise InputError(
+                    f"Invalid timeout: {opts.remote_timeout}. Must be "
+                    "an ISO 8601 duration or number of seconds."
+                ) from None
+        opts.remote_timeout = str(timeout)
+
+
 def prompt(workflows: Iterable[str]) -> None:
     """Ask user if they want to clean the given set of workflows."""
     print("Would clean the following workflows:")
@@ -197,28 +220,36 @@ async def run(*ids: str, opts: 'Values') -> None:
         workflows, multi_mode = await scan(workflows, multi_mode)
 
         if not workflows:
-            LOG.warning(f"No workflows matching {', '.join(ids)}")
+            LOG.warning(f"No stopped workflows matching {', '.join(ids)}")
             return
 
         workflows.sort()
         if multi_mode and not opts.skip_interactive:
             prompt(workflows)  # prompt for approval or exit
 
-    failed = {}
+    failed = False
     for workflow in workflows:
         try:
             init_clean(workflow, opts)
         except Exception as exc:
-            failed[workflow] = exc
+            failed = True
+            LOG.error(f"Failed to clean {workflow}\nError: {exc}")
+            if cylc.flow.flags.verbosity > 0:
+                LOG.exception(exc)
     if failed:
-        msg = "Clean failed:"
-        for workflow, exc_message in failed.items():
-            msg += f"\nWorkflow: {workflow}\nError: {exc_message}"
-        raise CylcError(msg)
+        raise CylcError("Clean failed")
 
 
 @cli_function(get_option_parser)
-def main(_, opts: 'Values', *ids: str):
+def main(_parser, opts: 'Values', *ids: str):
+    _main(opts, *ids)
+
+
+def _main(opts: 'Values', *ids: str):
+    """Run the clean command.
+
+    This is a separate function for ease of testing.
+    """
     if cylc.flow.flags.verbosity < 2:
         set_timestamps(LOG, False)
 
@@ -227,4 +258,6 @@ def main(_, opts: 'Values', *ids: str):
             "--local and --remote options are mutually exclusive"
         )
 
+    parse_timeout(opts)
+
     asyncio.run(run(*ids, opts=opts))
