@@ -21,20 +21,29 @@ Send messages to:
 - The scheduler, if communication is possible.
 """
 
-from logging import getLevelName, WARNING, ERROR, CRITICAL
+from logging import (
+    CRITICAL,
+    ERROR,
+    WARNING,
+    getLevelName,
+)
 import os
 import sys
 from typing import List
 
 from cylc.flow.exceptions import WorkflowStopped
 import cylc.flow.flags
-from cylc.flow.pathutil import get_workflow_run_job_dir
 from cylc.flow.network.client_factory import (
     CommsMeth,
     get_client,
-    get_comms_method
+    get_comms_method,
+)
+from cylc.flow.pathutil import get_workflow_run_job_dir
+from cylc.flow.task_outputs import (
+    TASK_OUTPUT_FAILED,
+    TASK_OUTPUT_STARTED,
+    TASK_OUTPUT_SUCCEEDED,
 )
-from cylc.flow.task_outputs import TASK_OUTPUT_STARTED, TASK_OUTPUT_SUCCEEDED
 from cylc.flow.wallclock import get_current_time_string
 
 
@@ -44,9 +53,9 @@ CYLC_JOB_EXIT = "CYLC_JOB_EXIT"
 CYLC_JOB_EXIT_TIME = "CYLC_JOB_EXIT_TIME"
 CYLC_MESSAGE = "CYLC_MESSAGE"
 
-ABORT_MESSAGE_PREFIX = "aborted/"
-FAIL_MESSAGE_PREFIX = "failed/"
-VACATION_MESSAGE_PREFIX = "vacated/"
+ABORT_MESSAGE_PREFIX = "aborted"
+FAIL_MESSAGE_PREFIX = TASK_OUTPUT_FAILED
+VACATION_MESSAGE_PREFIX = "vacated"
 
 STDERR_LEVELS = (getLevelName(level) for level in (WARNING, ERROR, CRITICAL))
 
@@ -69,6 +78,20 @@ mutation (
 '''
 
 
+def split_run_signal(message: str) -> tuple[str, str | None]:
+    """Get the run signal from a message.
+
+    >>> split_run_signal('failed/ERR')
+    ('failed', 'ERR')
+    >>> split_run_signal('aborted/mission')
+    ('aborted', 'mission')
+    >>> split_run_signal('failed')
+    ('failed', None)
+    """
+    prefix, *signal = message.split('/', 1)
+    return prefix, signal[0] if signal else None
+
+
 def record_messages(workflow: str, job_id: str, messages: List[list]) -> None:
     """Record task job messages.
 
@@ -102,7 +125,9 @@ def write_messages(workflow, job_id, messages, event_time):
     _append_job_status_file(workflow, job_id, event_time, messages)
 
 
-def send_messages(workflow, job_id, messages, event_time):
+def send_messages(
+    workflow: str, job_id: str, messages: List[list], event_time: str
+) -> None:
     workflow = os.path.normpath(workflow)
     try:
         pclient = get_client(workflow)
@@ -111,23 +136,24 @@ def send_messages(workflow, job_id, messages, event_time):
         # either the workflow is stopped or the contact file is not present
         # on the job host (i.e. comms method is polling)
         # eitherway don't try messaging
-        pass
-    except Exception:
-        # Backward communication not possible
+        return
+    except Exception as exc:
+        print(f"{type(exc).__name__}: {exc}", file=sys.stderr)
         if cylc.flow.flags.verbosity > 1:
             import traceback
             traceback.print_exc()
-    else:
-        mutation_kwargs = {
-            'request_string': MUTATION,
-            'variables': {
-                'wFlows': [workflow],
-                'taskJob': job_id,
-                'eventTime': event_time,
-                'messages': messages,
-            }
+        # cylc message shouldn't fail if the client can't initialize.
+        return
+    mutation_kwargs = {
+        'request_string': MUTATION,
+        'variables': {
+            'wFlows': [workflow],
+            'taskJob': job_id,
+            'eventTime': event_time,
+            'messages': messages,
         }
-        pclient('graphql', mutation_kwargs)
+    }
+    pclient('graphql', mutation_kwargs)
 
 
 def _append_job_status_file(workflow, job_id, event_time, messages):
@@ -138,12 +164,14 @@ def _append_job_status_file(workflow, job_id, event_time, messages):
     try:
         job_status_file = open(job_log_name + '.status', 'a')  # noqa: SIM115
         # TODO: niceify read/write/appending messages to this file
-    except IOError:
+    except IOError as exc:
+        print(f"{type(exc).__name__}: {exc}", file=sys.stderr)
         if cylc.flow.flags.verbosity > 1:
             import traceback
             traceback.print_exc()
         return
     for severity, message in messages:
+        message_prefix, signal = split_run_signal(message)
         if message == TASK_OUTPUT_STARTED:
             job_id = os.getppid()
             if job_id > 1:
@@ -155,19 +183,14 @@ def _append_job_status_file(workflow, job_id, event_time, messages):
             job_status_file.write(
                 ('%s=%s\n' % (CYLC_JOB_EXIT, TASK_OUTPUT_SUCCEEDED.upper())) +
                 ('%s=%s\n' % (CYLC_JOB_EXIT_TIME, event_time)))
-        elif message.startswith(FAIL_MESSAGE_PREFIX):
+        elif signal is not None and message_prefix in {
+            FAIL_MESSAGE_PREFIX, ABORT_MESSAGE_PREFIX
+        }:
             job_status_file.write(
-                ('%s=%s\n' % (
-                    CYLC_JOB_EXIT,
-                    message[len(FAIL_MESSAGE_PREFIX):])) +
-                ('%s=%s\n' % (CYLC_JOB_EXIT_TIME, event_time)))
-        elif message.startswith(ABORT_MESSAGE_PREFIX):
-            job_status_file.write(
-                ('%s=%s\n' % (
-                    CYLC_JOB_EXIT,
-                    message[len(ABORT_MESSAGE_PREFIX):])) +
-                ('%s=%s\n' % (CYLC_JOB_EXIT_TIME, event_time)))
-        elif message.startswith(VACATION_MESSAGE_PREFIX):
+                ('%s=%s\n' % (CYLC_JOB_EXIT, signal)) +
+                ('%s=%s\n' % (CYLC_JOB_EXIT_TIME, event_time))
+            )
+        elif signal is not None and message_prefix == VACATION_MESSAGE_PREFIX:
             # Job vacated, remove entries related to current job
             job_status_file_name = job_status_file.name
             job_status_file.close()
