@@ -23,35 +23,66 @@ This module provides the logic to:
 * Manage existing run database files on restart.
 """
 
+from collections import defaultdict
 import json
 import os
-from pkg_resources import parse_version
-from shutil import copy, rmtree
+from shutil import (
+    copy,
+    rmtree,
+)
 from sqlite3 import OperationalError
 from tempfile import mkstemp
 from typing import (
-    Any, AnyStr, Dict, List, Optional, Set, TYPE_CHECKING, Tuple, Union
+    TYPE_CHECKING,
+    Any,
+    AnyStr,
+    DefaultDict,
+    Dict,
+    List,
+    Optional,
+    Set,
+    Tuple,
+    Union,
 )
 
-from cylc.flow import LOG
+from packaging.version import parse as parse_version
+
+from cylc.flow import (
+    LOG,
+    __version__ as CYLC_VERSION,
+)
 from cylc.flow.broadcast_report import get_broadcast_change_iter
+from cylc.flow.exceptions import (
+    CylcError,
+    ServiceFileError,
+)
 from cylc.flow.rundb import CylcWorkflowDAO
-from cylc.flow import __version__ as CYLC_VERSION
-from cylc.flow.wallclock import get_current_time_string, get_utc_mode
-from cylc.flow.exceptions import CylcError, ServiceFileError
-from cylc.flow.util import serialise
+from cylc.flow.task_outputs import TASK_OUTPUT_SUCCEEDED
+from cylc.flow.util import (
+    deserialise_set,
+    serialise_set,
+)
+from cylc.flow.wallclock import (
+    get_current_time_string,
+    get_utc_mode,
+)
+from cylc.flow.scripts.set import XTRIGGER_PREREQ_PREFIX
 
 if TYPE_CHECKING:
     from pathlib import Path
-    from cylc.flow.cycling import PointBase
-    from cylc.flow.scheduler import Scheduler
-    from cylc.flow.task_pool import TaskPool
 
-Version = Any
-# TODO: narrow down Any (should be str | int) after implementing type
-# annotations in cylc.flow.task_state.TaskState
-DbArgDict = Dict[str, Any]
-DbUpdateTuple = Tuple[DbArgDict, DbArgDict]
+    from packaging.version import Version
+
+    from cylc.flow.cycling import PointBase
+    from cylc.flow.flow_mgr import FlowNums
+    from cylc.flow.rundb import (
+        DbArgDict,
+        DbUpdateTuple,
+    )
+    from cylc.flow.scheduler import Scheduler
+    from cylc.flow.task_events_mgr import EventKey
+    from cylc.flow.task_pool import TaskPool
+    from cylc.flow.task_proxy import TaskProxy
 
 
 PERM_PRIVATE = 0o600  # -rw-------
@@ -138,7 +169,9 @@ class WorkflowDatabaseManager:
             self.TABLE_TASKS_TO_HOLD: [],
             self.TABLE_XTRIGGERS: [],
             self.TABLE_ABS_OUTPUTS: []}
-        self.db_updates_map: Dict[str, List[DbUpdateTuple]] = {}
+        self.db_updates_map: DefaultDict[
+            str, List[DbUpdateTuple]
+        ] = defaultdict(list)
 
     def copy_pri_to_pub(self) -> None:
         """Copy content of primary database file to public database file."""
@@ -229,29 +262,23 @@ class WorkflowDatabaseManager:
         # Record workflow parameters and tasks in pool
         # Record any broadcast settings to be dumped out
         if any(self.db_deletes_map.values()):
-            for table_name, db_deletes in sorted(
-                    self.db_deletes_map.items()):
+            for table_name, db_deletes in sorted(self.db_deletes_map.items()):
                 while db_deletes:
                     where_args = db_deletes.pop(0)
                     self.pri_dao.add_delete_item(table_name, where_args)
                     self.pub_dao.add_delete_item(table_name, where_args)
         if any(self.db_inserts_map.values()):
-            for table_name, db_inserts in sorted(
-                    self.db_inserts_map.items()):
+            for table_name, db_inserts in sorted(self.db_inserts_map.items()):
                 while db_inserts:
                     db_insert = db_inserts.pop(0)
                     self.pri_dao.add_insert_item(table_name, db_insert)
                     self.pub_dao.add_insert_item(table_name, db_insert)
-        if (hasattr(self, 'db_updates_map') and
-                any(self.db_updates_map.values())):
-            for table_name, db_updates in sorted(
-                    self.db_updates_map.items()):
+        if any(self.db_updates_map.values()):
+            for table_name, db_updates in sorted(self.db_updates_map.items()):
                 while db_updates:
-                    set_args, where_args = db_updates.pop(0)
-                    self.pri_dao.add_update_item(
-                        table_name, set_args, where_args)
-                    self.pub_dao.add_update_item(
-                        table_name, set_args, where_args)
+                    db_update = db_updates.pop(0)
+                    self.pri_dao.add_update_item(table_name, db_update)
+                    self.pub_dao.add_update_item(table_name, db_update)
 
         # Previously, we used a separate thread for database writes. This has
         # now been removed. For the private database, there is no real
@@ -324,8 +351,16 @@ class WorkflowDatabaseManager:
             {"key": self.KEY_STOP_CLOCK_TIME, "value": schd.stop_clock_time},
             {"key": self.KEY_STOP_TASK, "value": schd.stop_task},
         ])
-        for key in (
+
+        # Store raw initial cycle point in the DB.
+        value = schd.config.evaluated_icp
+        value = None if value == 'reload' else value
+        self.put_workflow_params_1(
             self.KEY_INITIAL_CYCLE_POINT,
+            value or str(schd.config.initial_point)
+        )
+
+        for key in (
             self.KEY_FINAL_CYCLE_POINT,
             self.KEY_START_CYCLE_POINT,
             self.KEY_STOP_CYCLE_POINT
@@ -333,11 +368,15 @@ class WorkflowDatabaseManager:
             value = getattr(schd.options, key, None)
             value = None if value == 'reload' else value
             self.put_workflow_params_1(key, value)
-        for key in (
+
+        self.put_workflow_params_1(
+            self.KEY_CYCLE_POINT_TIME_ZONE,
+            getattr(schd.options, self.KEY_CYCLE_POINT_TIME_ZONE, None),
+        )
+        self.put_workflow_params_1(
             self.KEY_RUN_MODE,
-            self.KEY_CYCLE_POINT_TIME_ZONE
-        ):
-            self.put_workflow_params_1(key, getattr(schd.options, key, None))
+            schd.get_run_mode().value,
+        )
 
     def put_workflow_params_1(
         self, key: str, value: Union[AnyStr, float, None]
@@ -384,16 +423,17 @@ class WorkflowDatabaseManager:
             for key, value in template_vars.items()
         )
 
-    def put_task_event_timers(self, task_events_mgr):
+    def put_task_event_timers(self, task_events_mgr) -> None:
         """Put statements to update the task_action_timers table."""
         if task_events_mgr.event_timers_updated:
             self.db_deletes_map[self.TABLE_TASK_ACTION_TIMERS].append({})
-            for key, timer in task_events_mgr._event_timers.items():
-                key1, point, name, submit_num = key
+            id_key: 'EventKey'
+            for id_key, timer in task_events_mgr._event_timers.items():
+                key1 = (id_key.handler, id_key.event)
                 self.db_inserts_map[self.TABLE_TASK_ACTION_TIMERS].append({
-                    "name": name,
-                    "cycle": point,
-                    "ctx_key": json.dumps((key1, submit_num,)),
+                    "name": id_key.tokens['task'],
+                    "cycle": id_key.tokens['cycle'],
+                    "ctx_key": json.dumps((key1, int(id_key.tokens['job']),)),
                     "ctx": self._namedtuple2json(timer.ctx),
                     "delays": json.dumps(timer.delays),
                     "num": timer.num,
@@ -403,14 +443,14 @@ class WorkflowDatabaseManager:
             task_events_mgr.event_timers_updated = False
 
     def put_xtriggers(self, sat_xtrig):
-        """Put statements to update external triggers table."""
-        self.db_deletes_map[self.TABLE_XTRIGGERS].append({})
+        """Put statements to update xtriggers table."""
         for sig, res in sat_xtrig.items():
-            self.db_inserts_map[self.TABLE_XTRIGGERS].append({
-                "signature": sig,
-                "results": json.dumps(res)})
+            if not sig.startswith('wall_clock('):
+                self.db_inserts_map[self.TABLE_XTRIGGERS].append({
+                    "signature": sig,
+                    "results": json.dumps(res)})
 
-    def put_update_task_state(self, itask):
+    def put_update_task_state(self, itask: 'TaskProxy') -> None:
         """Update task_states table for current state of itask.
 
         NOTE the task_states table is normally updated along with the task pool
@@ -421,15 +461,37 @@ class WorkflowDatabaseManager:
             "time_updated": itask.state.time_updated,
             "status": itask.state.status,
             "flow_wait": itask.flow_wait,
-            "is_manual_submit": itask.is_manual_submit
+            "is_manual_submit": itask.is_manual_submit,
+        }
+        if not itask.transient:
+            set_args['submit_num'] = itask.submit_num
+        # Note tasks_states table rows are for latest submit_num only
+        # (not one row per submit).
+        where_args = {
+            "cycle": str(itask.point),
+            "name": itask.tdef.name,
+            "flow_nums": serialise_set(itask.flow_nums),
+        }
+        self.db_updates_map[self.TABLE_TASK_STATES].append(
+            (set_args, where_args)
+        )
+
+    def put_update_task_flow_wait(self, itask):
+        """Update flow_wait status of a task, in the task_states table.
+
+        NOTE the task_states table is normally updated along with the task pool
+        table. This method is only needed as a final update for a non-pool task
+        that just spawned its children after a flow wait.
+        """
+        set_args = {
+            "time_updated": itask.state.time_updated,
+            "flow_wait": itask.flow_wait,
         }
         where_args = {
             "cycle": str(itask.point),
             "name": itask.tdef.name,
-            "flow_nums": serialise(itask.flow_nums),
-            "submit_num": itask.submit_num,
+            "flow_nums": serialise_set(itask.flow_nums),
         }
-        self.db_updates_map.setdefault(self.TABLE_TASK_STATES, [])
         self.db_updates_map[self.TABLE_TASK_STATES].append(
             (set_args, where_args))
 
@@ -451,22 +513,30 @@ class WorkflowDatabaseManager:
         # This should already be done by self.put_task_event_timers above:
         # self.db_deletes_map[self.TABLE_TASK_ACTION_TIMERS].append({})
         self.db_deletes_map[self.TABLE_TASK_TIMEOUT_TIMERS].append({})
-        for itask in pool.get_all_tasks():
+        for itask in pool.get_tasks():
             for prereq in itask.state.prerequisites:
                 for (p_cycle, p_name, p_output), satisfied_state in (
-                    prereq.satisfied.items()
+                    prereq.items()
                 ):
                     self.put_insert_task_prerequisites(itask, {
-                        "flow_nums": serialise(itask.flow_nums),
                         "prereq_name": p_name,
                         "prereq_cycle": p_cycle,
                         "prereq_output": p_output,
                         "satisfied": satisfied_state
                     })
+            for x_label, x_satisfied in itask.state.xtriggers.items():
+                if x_satisfied:
+                    self.put_insert_task_prerequisites(itask, {
+                        "prereq_name": x_label,
+                        "prereq_cycle": XTRIGGER_PREREQ_PREFIX,
+                        "prereq_output": TASK_OUTPUT_SUCCEEDED,
+                        "satisfied": True
+                    })
+
             self.db_inserts_map[self.TABLE_TASK_POOL].append({
                 "name": itask.tdef.name,
                 "cycle": str(itask.point),
-                "flow_nums": serialise(itask.flow_nums),
+                "flow_nums": serialise_set(itask.flow_nums),
                 "status": itask.state.status,
                 "is_held": itask.state.is_held
             })
@@ -505,14 +575,14 @@ class WorkflowDatabaseManager:
                     "time_updated": itask.state.time_updated,
                     "submit_num": itask.submit_num,
                     "try_num": itask.get_try_num(),
-                    "status": itask.state.status
+                    "status": itask.state.status,
+                    "is_manual_submit": itask.is_manual_submit,
                 }
                 where_args = {
                     "cycle": str(itask.point),
                     "name": itask.tdef.name,
-                    "flow_nums": serialise(itask.flow_nums)
+                    "flow_nums": serialise_set(itask.flow_nums)
                 }
-                self.db_updates_map.setdefault(self.TABLE_TASK_STATES, [])
                 self.db_updates_map[self.TABLE_TASK_STATES].append(
                     (set_args, where_args)
                 )
@@ -546,12 +616,25 @@ class WorkflowDatabaseManager:
         """Put INSERT statement for task_jobs table."""
         self._put_insert_task_x(CylcWorkflowDAO.TABLE_TASK_JOBS, itask, args)
 
-    def put_insert_task_states(self, itask, args):
+    def put_insert_task_states(self, itask: 'TaskProxy') -> None:
         """Put INSERT statement for task_states table."""
-        self._put_insert_task_x(CylcWorkflowDAO.TABLE_TASK_STATES, itask, args)
+        now = get_current_time_string()
+        self._put_insert_task_x(
+            CylcWorkflowDAO.TABLE_TASK_STATES,
+            itask,
+            {
+                "time_created": now,
+                "time_updated": now,
+                "status": itask.state.status,
+                "flow_nums": serialise_set(itask.flow_nums),
+                "flow_wait": itask.flow_wait,
+                "is_manual_submit": itask.is_manual_submit,
+            },
+        )
 
     def put_insert_task_prerequisites(self, itask, args):
         """Put INSERT statement for task_prerequisites table."""
+        args.setdefault("flow_nums", serialise_set(itask.flow_nums))
         self._put_insert_task_x(self.TABLE_TASK_PREREQUISITES, itask, args)
 
     def put_insert_task_outputs(self, itask):
@@ -560,8 +643,8 @@ class WorkflowDatabaseManager:
             CylcWorkflowDAO.TABLE_TASK_OUTPUTS,
             itask,
             {
-                "flow_nums": serialise(itask.flow_nums),
-                "outputs": json.dumps([])
+                "flow_nums": serialise_set(itask.flow_nums),
+                "outputs": json.dumps({})
             }
         )
 
@@ -588,50 +671,137 @@ class WorkflowDatabaseManager:
             }
         )
 
-    def _put_insert_task_x(self, table_name, itask, args):
+    def _put_insert_task_x(
+        self, table_name: str, itask: 'TaskProxy', args: 'DbArgDict'
+    ) -> None:
         """Put INSERT statement for a task_* table."""
         args.update({
             "name": itask.tdef.name,
-            "cycle": str(itask.point)})
-        if "submit_num" not in args:
-            args["submit_num"] = itask.submit_num
-        self.db_inserts_map.setdefault(table_name, [])
-        self.db_inserts_map[table_name].append(args)
+            "cycle": str(itask.point),
+        })
+        args.setdefault("submit_num", itask.submit_num)
+        self.db_inserts_map.setdefault(table_name, []).append(args)
 
-    def put_update_task_jobs(self, itask, set_args):
+    def put_update_task_jobs(self, itask: 'TaskProxy', set_args: dict) -> None:
         """Put UPDATE statement for task_jobs table."""
+        set_args.setdefault('flow_nums', serialise_set(itask.flow_nums))
         self._put_update_task_x(
-            CylcWorkflowDAO.TABLE_TASK_JOBS, itask, set_args)
+            CylcWorkflowDAO.TABLE_TASK_JOBS, itask, set_args
+        )
 
-    def put_update_task_outputs(self, itask):
+    def put_update_task_outputs(self, itask: 'TaskProxy') -> None:
         """Put UPDATE statement for task_outputs table."""
-        outputs = []
-        for _, message in itask.state.outputs.get_completed_all():
-            outputs.append(message)
         set_args = {
-            "outputs": json.dumps(outputs)
+            "outputs": json.dumps(
+                itask.state.outputs.get_completed_outputs()
+            )
         }
         where_args = {
             "cycle": str(itask.point),
             "name": itask.tdef.name,
-            "flow_nums": serialise(itask.flow_nums),
+            "flow_nums": serialise_set(itask.flow_nums),
         }
-        self.db_updates_map.setdefault(self.TABLE_TASK_OUTPUTS, [])
         self.db_updates_map[self.TABLE_TASK_OUTPUTS].append(
-            (set_args, where_args))
+            (set_args, where_args)
+        )
 
-    def _put_update_task_x(self, table_name, itask, set_args):
+    def _put_update_task_x(
+        self, table_name: str, itask: 'TaskProxy', set_args: 'DbArgDict'
+    ) -> None:
         """Put UPDATE statement for a task_* table."""
-        where_args = {
+        where_args: Dict[str, Any] = {
             "cycle": str(itask.point),
-            "name": itask.tdef.name}
+            "name": itask.tdef.name,
+        }
         if "submit_num" not in set_args:
             where_args["submit_num"] = itask.submit_num
         if "flow_nums" not in set_args:
-            where_args["flow_nums"] = serialise(itask.flow_nums)
-        self.db_updates_map.setdefault(table_name, [])
+            where_args["flow_nums"] = serialise_set(itask.flow_nums)
         self.db_updates_map[table_name].append((set_args, where_args))
 
+    def remove_task_from_flows(
+        self, point: str, name: str, flow_nums: 'FlowNums'
+    ) -> 'FlowNums':
+        """Remove flow numbers for a task in the task_states and task_outputs
+        tables.
+
+        Args:
+            point: Cycle point of the task.
+            name: Name of the task.
+            flow_nums: Flow numbers to remove. If empty, remove all
+                flow numbers.
+
+        Returns the flow numbers that were removed, if any.
+
+        N.B. the task_prerequisites table is automatically updated separately
+        during the main loop.
+        """
+        removed_flow_nums: FlowNums = set()
+        for table in (
+            self.TABLE_TASK_STATES,
+            self.TABLE_TASK_OUTPUTS,
+        ):
+            fnums_select_stmt = rf'''
+                SELECT
+                    flow_nums
+                FROM
+                    {table}
+                WHERE
+                    cycle = ?
+                    AND name = ?
+            '''  # nosec B608 (table name is a code constant)
+            fnums_select_cursor = self.pri_dao.connect().execute(
+                fnums_select_stmt, (point, name)
+            )
+
+            if not flow_nums:
+                for db_fnums_str, *_ in fnums_select_cursor:
+                    removed_flow_nums.update(deserialise_set(db_fnums_str))
+
+                stmt = rf'''
+                    UPDATE OR REPLACE
+                        {table}
+                    SET
+                        flow_nums = ?
+                    WHERE
+                        cycle = ?
+                        AND name = ?
+                '''  # nosec B608 (table name is a code constant)
+                params: List[tuple] = [(serialise_set(), point, name)]
+            else:
+                # Mapping of existing flow nums to what should be left after
+                # removing the specified flow nums:
+                flow_nums_map: Dict[str, FlowNums] = {}
+                for db_fnums_str, *_ in fnums_select_cursor:
+                    db_fnums: FlowNums = deserialise_set(db_fnums_str)
+                    fnums_to_remove = db_fnums.intersection(flow_nums)
+                    if fnums_to_remove:
+                        flow_nums_map[db_fnums_str] = db_fnums.difference(
+                            flow_nums
+                        )
+                        removed_flow_nums.update(fnums_to_remove)
+
+                stmt = rf'''
+                    UPDATE OR REPLACE
+                        {table}
+                    SET
+                        flow_nums = ?
+                    WHERE
+                        cycle = ?
+                        AND name = ?
+                        AND flow_nums = ?
+                '''  # nosec B608 (table name is a code constant)
+                params = [
+                    (serialise_set(new), point, name, old)
+                    for old, new in flow_nums_map.items()
+                ]
+
+            self.db_updates_map[table].append(
+                (stmt, params)
+            )
+
+        return removed_flow_nums
+
     def recover_pub_from_pri(self):
         """Recover public database from private database."""
         if self.pub_dao.n_tries >= self.pub_dao.MAX_TRIES:
@@ -656,7 +826,7 @@ class WorkflowDatabaseManager:
             self.process_queued_ops()
 
     @classmethod
-    def _get_last_run_version(cls, pri_dao: CylcWorkflowDAO) -> Version:
+    def _get_last_run_version(cls, pri_dao: CylcWorkflowDAO) -> 'Version':
         """Return the version of Cylc this DB was last run with.
 
         Args:
@@ -672,11 +842,13 @@ class WorkflowDatabaseManager:
                         {cls.TABLE_WORKFLOW_PARAMS}
                     WHERE
                         key == ?
-                ''',  # nosec (table name is a code constant)
+                ''',  # nosec B608 (table name is a code constant)
                 [cls.KEY_CYLC_VERSION]
             ).fetchone()[0]
-        except (TypeError, OperationalError):
-            raise ServiceFileError(f"{INCOMPAT_MSG}, or is corrupted.")
+        except (TypeError, OperationalError) as exc:
+            raise ServiceFileError(
+                f"{INCOMPAT_MSG}, or is corrupted."
+            ) from exc
         return parse_version(last_run_ver)
 
     @classmethod
@@ -718,8 +890,7 @@ class WorkflowDatabaseManager:
 
         # We can't upgrade if the flow_nums in task_states are not
         # distinct.
-        from cylc.flow.util import deserialise
-        flow_nums = deserialise(conn.execute(
+        flow_nums = deserialise_set(conn.execute(
             'SELECT DISTINCT flow_nums FROM task_states;').fetchall()[0][0])
         if len(flow_nums) != 1:
             raise CylcError(
@@ -746,7 +917,7 @@ class WorkflowDatabaseManager:
                 cls.upgrade_pre_810(pri_dao)
 
     @classmethod
-    def check_db_compatibility(cls, db_file: Union['Path', str]) -> Version:
+    def check_db_compatibility(cls, db_file: Union['Path', str]) -> 'Version':
         """Check this DB is compatible with this Cylc version.
 
         Raises:
