@@ -22,30 +22,40 @@ Use the fixtures provided in the conftest instead.
 """
 
 import asyncio
-from pathlib import Path
-from async_timeout import timeout
-from contextlib import asynccontextmanager, contextmanager
+from contextlib import (
+    asynccontextmanager,
+    contextmanager,
+)
 import logging
+from pathlib import Path
+from secrets import token_hex
+from typing import (
+    Any,
+    Optional,
+    Union,
+)
+
 import pytest
-from typing import Any, Optional, Union
-from uuid import uuid1
 
 from cylc.flow import CYLC_LOG
-from cylc.flow.workflow_files import WorkflowFiles
-from cylc.flow.scheduler import Scheduler, SchedulerStop
+from cylc.flow.scheduler import (
+    Scheduler,
+    SchedulerStop,
+)
 from cylc.flow.scheduler_cli import RunOptions
+from cylc.flow.workflow_files import WorkflowFiles
 from cylc.flow.workflow_status import StopMode
 
 from .flow_writer import flow_config_str
 
 
-def _make_src_flow(src_path, conf):
+def _make_src_flow(src_path, conf, filename=WorkflowFiles.FLOW_FILE):
     """Construct a workflow on the filesystem"""
-    flow_src_dir = (src_path / str(uuid1()))
+    flow_src_dir = (src_path / token_hex(4))
     flow_src_dir.mkdir(parents=True, exist_ok=True)
     if isinstance(conf, dict):
         conf = flow_config_str(conf)
-    with open((flow_src_dir / WorkflowFiles.FLOW_FILE), 'w+') as flow_file:
+    with open((flow_src_dir / filename), 'w+') as flow_file:
         flow_file.write(conf)
     return flow_src_dir
 
@@ -55,22 +65,51 @@ def _make_flow(
     test_dir: Path,
     conf: Union[dict, str],
     name: Optional[str] = None,
-    id_: Optional[str] = None,
+    workflow_id: Optional[str] = None,
+    defaults: Optional[bool] = True,
+    filename: str = WorkflowFiles.FLOW_FILE,
 ) -> str:
-    """Construct a workflow on the filesystem."""
-    if id_:
-        flow_run_dir = (cylc_run_dir / id_)
+    """Construct a workflow on the filesystem.
+
+    Args:
+        conf: Either a workflow config dictionary, or a graph string to be
+            used as the R1 graph in the workflow config.
+        defaults: Set up a common defaults.
+            * [scheduling]allow implicit tasks = true
+
+            Set false for Cylc 7 upgrader tests.
+    """
+    if workflow_id:
+        flow_run_dir = (cylc_run_dir / workflow_id)
     else:
         if name is None:
-            name = str(uuid1())
+            name = token_hex(4)
         flow_run_dir = (test_dir / name)
     flow_run_dir.mkdir(parents=True, exist_ok=True)
-    id_ = str(flow_run_dir.relative_to(cylc_run_dir))
-    if isinstance(conf, dict):
-        conf = flow_config_str(conf)
-    with open((flow_run_dir / WorkflowFiles.FLOW_FILE), 'w+') as flow_file:
-        flow_file.write(conf)
-    return id_
+    workflow_id = str(flow_run_dir.relative_to(cylc_run_dir))
+    if isinstance(conf, str):
+        conf = {
+            'scheduling': {
+                'graph': {
+                    'R1': conf
+                }
+            }
+        }
+    if defaults:
+        # set the default simulation runtime to zero (can be overridden)
+        (
+            conf.setdefault('runtime', {})
+            .setdefault('root', {})
+            .setdefault('simulation', {})
+            .setdefault('default run length', 'PT0S')
+        )
+        # allow implicit tasks by default:
+        conf.setdefault('scheduler', {}).setdefault(
+            'allow implicit tasks', 'True')
+
+    with open((flow_run_dir / filename), 'w+') as flow_file:
+        flow_file.write(flow_config_str(conf))
+    return workflow_id
 
 
 @contextmanager
@@ -123,7 +162,7 @@ async def _start_flow(
         # context manager.
         # Need to shut down Scheduler, but time out in case something
         # goes wrong:
-        async with timeout(5):
+        async with asyncio.timeout(5):
             await schd.shutdown(SchedulerStop("integration test teardown"))
 
 
@@ -156,14 +195,14 @@ async def _run_flow(
         # context manager.
         # Need to shut down Scheduler, but time out in case something
         # goes wrong:
-        async with timeout(5):
+        async with asyncio.timeout(5):
             if task:
                 # ask the scheduler to shut down nicely,
                 # let main loop handle it:
                 schd._set_stop(StopMode.REQUEST_NOW_NOW)
                 await task
         if schd.contact_data:
-            async with timeout(5):
+            async with asyncio.timeout(5):
                 # Scheduler still running... try more forceful tear down:
                 await schd.shutdown(SchedulerStop("integration test teardown"))
         if task:
