@@ -32,105 +32,20 @@ from cylc.flow.parsec.exceptions import (
 )
 from cylc.flow.parsec.OrderedDict import OrderedDictWithDefaults
 from cylc.flow.parsec.fileparse import (
+    EXTRA_VARS_TEMPLATE,
     _prepend_old_templatevars,
     _get_fpath_for_source,
+    get_cylc_env_vars,
     addict,
     addsect,
     multiline,
     parse,
+    process_plugins,
     read_and_proc,
     merge_template_vars
 )
 
 
-def get_multiline():
-    """Data provider for multiline tests. Returned values are:
-
-    file_lines, value, index, maxline, error_message, expected
-    """
-    r = [
-        ([], "'''single line'''", 0, 0, None, ("'''single line'''", 0)),
-        (
-            ["'''single line"],
-            "'''single line",  # missing closing quote
-            0,
-            0,
-            FileParseError,
-            {
-                'reason': 'Multiline string not closed',
-                'line': "'''single line"
-            }
-        ),
-        (
-            ["'''", "single line"],
-            "'''\n   '''single line",  # missing closing quote
-            0,
-            0,
-            FileParseError,
-            {
-                'reason': 'Invalid line',
-                'line': "'''"
-            }
-        ),
-        (
-            ["", "another value"],  # multiline, but we forgot to close quotes
-            "'''a\n#b",
-            0,
-            1,
-            FileParseError,
-            {
-                'reason': "Multiline string not closed"
-            }
-        ),
-        (
-            ["", "c'''"],
-            "'''a\n#b",
-            0,
-            1,
-            None,
-            ("'''a\n#b\nc'''", 1)
-        ),
-        (
-            ["", "c'''"],  # multiline, but we forgot to close quotes
-            "'''a\n#b",
-            0,
-            10000,  # no error. The function will stop before on the quotes
-            None,
-            ("'''a\n#b\nc'''", 1)
-        ),
-        (
-            ["", "c", "hello", ""],  # quotes out of balance
-            "'''a\n#b",
-            0,
-            3,
-            FileParseError,
-            {
-                'reason': "Multiline string not closed"
-            }
-        ),
-        (
-            ["", "c", "hello", ""],
-            "'''a\n#b",
-            0,
-            4,  # one too many
-            IndexError,
-            None
-        ),
-        (
-            ["", "a'''c", "hello", ""],
-            "'''a\n#b",
-            0,
-            3,
-            FileParseError,
-            {
-                'reason': 'Invalid line',
-                'line': "a'''c"
-            }
-        )
-    ]
-    return r
-
-
 def test_file_parse_error():
     error = FileParseError(reason="No reason")
     assert str(error) == "No reason"
@@ -264,18 +179,106 @@ def test_addict_replace_value_1():
     assert cfg['scheduling']['graph']['team']['graph'] == ['ABC', 'test']
 
 
-def test_multiline():
-    for flines, value, index, maxline, exc, expected in get_multiline():
-        if exc is not None:
-            with pytest.raises(exc) as cm:
-                multiline(flines, value, index, maxline)
-            if isinstance(cm.value, FileParseError):
-                exc = cm.value
-                for key, attr in expected.items():
-                    assert getattr(exc, key) == attr
-        else:
-            r = multiline(flines, value, index, maxline)
-            assert r == expected
+@pytest.mark.parametrize(
+    'flines, value, index, maxline, exc, expected',
+    (
+        param(
+            [],
+            "'''single line'''",
+            0,
+            0,
+            None,
+            ("'''single line'''", 0),
+            id='single-line',
+        ),
+        param(
+            ["'''single line"],
+            "'''single line",  # missing closing quote
+            0,
+            0,
+            FileParseError,
+            {
+                'reason': 'Multiline string not closed',
+                'line': "'''single line",
+            },
+            id='missing-closing-quote',
+        ),
+        param(
+            ["'''", "single line"],
+            "'''\n   '''single line",  # missing closing quote
+            0,
+            0,
+            FileParseError,
+            {'reason': 'Invalid line', 'line': "'''"},
+            id='missing-closing-quote2',
+        ),
+        param(
+            ["", "another value"],  # multiline, but we forgot to close quotes
+            "'''a\n#b",
+            0,
+            1,
+            FileParseError,
+            {'reason': "Multiline string not closed"},
+            id='multiline-missing-closing-quote',
+        ),
+        param(
+            ["", "c'''"],
+            "'''a\n#b",
+            0,
+            1,
+            None,
+            ("'''a\n#b\nc'''", 1),
+            id='good-path',
+        ),
+        param(
+            ["", "c'''"],  # multiline, but we forgot to close quotes
+            "'''a\n#b",
+            0,
+            10000,  # no error. The function will stop before on the quotes
+            None,
+            ("'''a\n#b\nc'''", 1),
+            id='multiline-missing-closing-quote2',
+        ),
+        param(
+            ["", "c", "hello", ""],  # quotes out of balance
+            "'''a\n#b",
+            0,
+            3,
+            FileParseError,
+            {'reason': "Multiline string not closed"},
+            id='unbalanced-quotes',
+        ),
+        param(
+            ["", "c", "hello", ""],
+            "'''a\n#b",
+            0,
+            4,  # one too many
+            IndexError,
+            None,
+            id='one-too-many',
+        ),
+        param(
+            ["", "a'''c", "hello", ""],
+            "'''a\n#b",
+            0,
+            3,
+            FileParseError,
+            {'reason': 'Invalid line', 'line': "a'''c"},
+            id='invalid-quotes',
+        ),
+    ),
+)
+def test_multiline(flines, value, index, maxline, exc, expected):
+    if exc is not None:
+        with pytest.raises(exc) as cm:
+            multiline(flines, value, index, maxline)
+        if isinstance(cm.value, FileParseError):
+            exc = cm.value
+            for key, attr in expected.items():
+                assert getattr(exc, key) == attr
+    else:
+        r = multiline(flines, value, index, maxline)
+        assert r == expected
 
 
 def test_read_and_proc_no_template_engine():
@@ -283,8 +286,9 @@ def test_read_and_proc_no_template_engine():
         fpath = tf.name
         template_vars = None
         viewcfg = {
-            'empy': False, 'jinja2': False,
-            'contin': False, 'inline': False
+            'jinja2': False,
+            'contin': False,
+            'inline': False,
         }
         tf.write("a=b\n".encode())
         tf.flush()
@@ -297,8 +301,9 @@ def test_read_and_proc_no_template_engine():
         tf.flush()
 
         viewcfg = {
-            'empy': False, 'jinja2': False,
-            'contin': True, 'inline': False
+            'jinja2': False,
+            'contin': True,
+            'inline': False,
         }
         r = read_and_proc(fpath=fpath, template_vars=template_vars,
                           viewcfg=viewcfg)
@@ -310,9 +315,12 @@ def test_inline():
         fpath = tf.name
         template_vars = None
         viewcfg = {
-            'empy': False, 'jinja2': False,
-            'contin': False, 'inline': True,
-            'mark': None, 'single': None, 'label': None
+            'jinja2': False,
+            'contin': False,
+            'inline': True,
+            'mark': None,
+            'single': None,
+            'label': None,
         }
         with NamedTemporaryFile() as include_file:
             include_file.write("c=d".encode())
@@ -330,9 +338,12 @@ def test_inline_error():
         fpath = tf.name
         template_vars = None
         viewcfg = {
-            'empy': False, 'jinja2': False,
-            'contin': False, 'inline': True,
-            'mark': None, 'single': None, 'label': None
+            'jinja2': False,
+            'contin': False,
+            'inline': True,
+            'mark': None,
+            'single': None,
+            'label': None,
         }
         tf.write("a=b\n%include \"404.txt\"".encode())
         tf.flush()
@@ -349,8 +360,9 @@ def test_read_and_proc_jinja2():
             'name': 'Cylc'
         }
         viewcfg = {
-            'empy': False, 'jinja2': True,
-            'contin': False, 'inline': False
+            'jinja2': True,
+            'contin': False,
+            'inline': False,
         }
         tf.write("#!jinja2\na={{ name }}\n".encode())
         tf.flush()
@@ -372,7 +384,6 @@ def test_read_and_proc_cwd(tmp_path):
         (sdir / sub).touch()
 
     viewcfg = {
-        'empy': False,
         'jinja2': True,
         'contin': False,
         'inline': False
@@ -402,8 +413,9 @@ def test_read_and_proc_jinja2_error():
             'name': 'Cylc'
         }
         viewcfg = {
-            'empy': False, 'jinja2': True,
-            'contin': False, 'inline': False
+            'jinja2': True,
+            'contin': False,
+            'inline': False,
         }
         tf.write("#!jinja2\na={{ name \n".encode())
         tf.flush()
@@ -423,8 +435,9 @@ def test_read_and_proc_jinja2_error_missing_shebang():
             'name': 'Cylc'
         }
         viewcfg = {
-            'empy': False, 'jinja2': True,
-            'contin': False, 'inline': False
+            'jinja2': True,
+            'contin': False,
+            'inline': False,
         }
         # first line is missing shebang!
         tf.write("a={{ name }}\n".encode())
@@ -434,8 +447,6 @@ def test_read_and_proc_jinja2_error_missing_shebang():
         assert r == ['a={{ name }}']
 
 
-# --- originally we had a test for empy here, moved to test_empysupport
-
 def test_parse_keys_only_singleline():
     with NamedTemporaryFile() as of, NamedTemporaryFile() as tf:
         fpath = tf.name
@@ -737,9 +748,10 @@ def test_get_fpath_for_source(tmp_path):
     assert _get_fpath_for_source(
         rundir / 'flow.cylc', opts) == str(srcdir / 'flow.cylc')
 
+
 def test_user_has_no_cwd(tmp_path):
     """Test we can parse a config file even if cwd does not exist."""
-    cwd = tmp_path/"cwd"
+    cwd = tmp_path / "cwd"
     os.mkdir(cwd)
     os.chdir(cwd)
     os.rmdir(cwd)
@@ -754,3 +766,51 @@ def test_user_has_no_cwd(tmp_path):
         tf.flush()
         # Should not raise FileNotFoundError from os.getcwd():
         parse(fpath=fpath, output_fname="")
+
+
+def test_get_cylc_env_vars(monkeypatch):
+    """It should return CYLC env vars but not CYLC_VERSION or CYLC_ENV_NAME."""
+    monkeypatch.setattr(
+        'os.environ',
+        {
+            "CYLC_VERSION": "betwixt",
+            "CYLC_ENV_NAME": "between",
+            "CYLC_QUESTION": "que?",
+            "CYLC_ANSWER": "42",
+            "FOO": "foo"
+        }
+    )
+    assert (
+        get_cylc_env_vars() == {
+            "CYLC_QUESTION": "que?",
+            "CYLC_ANSWER": "42",
+        }
+    )
+
+
+class EntryPointWrapper:
+    """Wraps a method to make it look like an entry point."""
+
+    def __init__(self, fcn):
+        self.name = fcn.__name__
+        self.fcn = fcn
+
+    def load(self):
+        return self.fcn
+
+
+@EntryPointWrapper
+def pre_configure_basic(*_, **__):
+    """Simple plugin that returns one env var and one template var."""
+    return {'env': {'foo': 44}, 'template_variables': {}}
+
+
+def test_plugins_not_called_on_global_config(monkeypatch):
+    monkeypatch.setattr(
+        'cylc.flow.plugins.iter_entry_points',
+        lambda x: [pre_configure_basic]
+    )
+    result = process_plugins('/pennine/way/flow.cylc', {})
+    assert result != EXTRA_VARS_TEMPLATE
+    result = process_plugins('/appalachian/trail/global.cylc', {})
+    assert result == EXTRA_VARS_TEMPLATE
