@@ -14,7 +14,6 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from os import read
 import pytest
 
 from cylc.flow.parsec import fileparse
@@ -25,8 +24,8 @@ from cylc.flow.parsec.exceptions import TemplateVarLanguageClash
 @pytest.mark.parametrize(
     'templating, hashbang',
     [
-        ['empy', 'jinja2'],
-        ['jinja2', 'empy']
+        ['other', 'jinja2'],
+        ['jinja2', 'other']
     ]
 )
 def test_read_and_proc_raises_TemplateVarLanguageClash(
