@@ -48,7 +48,7 @@ def generate_spec():
             cylc.flow.parsec.config.ConfigNode
 
         """
-        with Conf('/') as myconf:  # noqa: SIM117
+        with Conf('/') as myconf:
             with Conf(typ):
                 Conf('<item>', validator)
         return myconf
