@@ -19,7 +19,9 @@ import os
 import shutil
 from glob import iglob
 from pathlib import Path
+from subprocess import Popen
 from typing import (
+    TYPE_CHECKING,
     Any,
     Callable,
     Dict,
@@ -33,8 +35,7 @@ from unittest import mock
 
 import pytest
 
-from cylc.flow import CYLC_LOG
-from cylc.flow import clean as cylc_clean
+from cylc.flow import CYLC_LOG, clean as cylc_clean
 from cylc.flow.clean import (
     _clean_using_glob,
     _remote_clean_cmd,
@@ -43,7 +44,7 @@ from cylc.flow.clean import (
     init_clean,
 )
 from cylc.flow.exceptions import (
-    ContactFileExists,
+    SchedulerAlive,
     CylcError,
     InputError,
     PlatformError,
@@ -57,16 +58,21 @@ from cylc.flow.workflow_files import (
     get_symlink_dirs,
 )
 
-from .conftest import MonkeyMock
 from .filetree import (
     FILETREE_1,
     FILETREE_2,
     FILETREE_3,
     FILETREE_4,
+    Symlink,
     create_filetree,
     get_filetree_as_list,
 )
 
+
+if TYPE_CHECKING:
+    from .conftest import MonkeyMock
+
+
 NonCallableFixture = Any
 
 
@@ -113,7 +119,7 @@ def test_clean_check__fail(
     """
     def mocked_detect_old_contact_file(*a, **k):
         if not stopped:
-            raise ContactFileExists('Mocked error')
+            raise SchedulerAlive('Mocked error')
 
     monkeypatch.setattr(
         'cylc.flow.clean.detect_old_contact_file',
@@ -151,7 +157,7 @@ def test_init_clean(
     opts: Dict[str, Any],
     clean_called: bool,
     remote_clean_called: bool,
-    monkeypatch: pytest.MonkeyPatch, monkeymock: MonkeyMock,
+    monkeypatch: pytest.MonkeyPatch, monkeymock: 'MonkeyMock',
     tmp_run_dir: Callable
 ) -> None:
     """Test the init_clean() function logic.
@@ -176,12 +182,11 @@ def test_init_clean(
 
 
 def test_init_clean__no_dir(
-    monkeymock: MonkeyMock, tmp_run_dir: Callable,
+    monkeymock: 'MonkeyMock', tmp_run_dir: Callable,
     caplog: pytest.LogCaptureFixture
 ) -> None:
     """Test init_clean() when the run dir doesn't exist"""
     caplog.set_level(logging.INFO, CYLC_LOG)
-    tmp_run_dir()
     mock_clean = monkeymock('cylc.flow.clean.clean')
     mock_remote_clean = monkeymock('cylc.flow.clean.remote_clean')
 
@@ -192,7 +197,7 @@ def test_init_clean__no_dir(
 
 
 def test_init_clean__no_db(
-    monkeymock: MonkeyMock, tmp_run_dir: Callable,
+    monkeymock: 'MonkeyMock', tmp_run_dir: Callable,
     caplog: pytest.LogCaptureFixture
 ) -> None:
     """Test init_clean() when the workflow database doesn't exist"""
@@ -210,7 +215,7 @@ def test_init_clean__no_db(
 
 
 def test_init_clean__remote_only_no_db(
-    monkeymock: MonkeyMock, tmp_run_dir: Callable
+    monkeymock: 'MonkeyMock', tmp_run_dir: Callable
 ) -> None:
     """Test remote-only init_clean() when the workflow DB doesn't exist"""
     tmp_run_dir('hoth')
@@ -231,7 +236,7 @@ def test_init_clean__running_workflow(
 ) -> None:
     """Test init_clean() fails when workflow is still running"""
     def mock_err(*args, **kwargs):
-        raise ContactFileExists("Mocked error")
+        raise SchedulerAlive("Mocked error")
     monkeypatch.setattr('cylc.flow.clean.detect_old_contact_file',
                         mock_err)
     tmp_run_dir('yavin')
@@ -250,7 +255,7 @@ def test_init_clean__rm_dirs(
     rm_dirs: Optional[List[str]],
     expected_clean: Set[str],
     expected_remote_clean: List[str],
-    monkeymock: MonkeyMock, monkeypatch: pytest.MonkeyPatch,
+    monkeymock: 'MonkeyMock', monkeypatch: pytest.MonkeyPatch,
     tmp_run_dir: Callable
 ) -> None:
     """Test init_clean() with the --rm option.
@@ -263,7 +268,9 @@ def test_init_clean__rm_dirs(
     """
     id_ = 'dagobah'
     run_dir: Path = tmp_run_dir(id_)
-    Path(run_dir, WorkflowFiles.Service.DIRNAME, WorkflowFiles.Service.DB).touch()
+    Path(
+        run_dir, WorkflowFiles.Service.DIRNAME, WorkflowFiles.Service.DB
+    ).touch()
     mock_clean = monkeymock('cylc.flow.clean.clean')
     mock_remote_clean = monkeymock('cylc.flow.clean.remote_clean')
     platforms = {'platform_one'}
@@ -274,7 +281,8 @@ def test_init_clean__rm_dirs(
     init_clean(id_, opts=opts)
     mock_clean.assert_called_with(id_, run_dir, expected_clean)
     mock_remote_clean.assert_called_with(
-        id_, platforms, expected_remote_clean, opts.remote_timeout)
+        id_, platforms, opts.remote_timeout, expected_remote_clean
+    )
 
 
 @pytest.mark.parametrize(
@@ -535,7 +543,7 @@ def filetree_for_testing_cylc_clean(tmp_path: Path):
                 'cylc-run': {'foo': {'bar': {
                     '.service': {'db': None},
                     'flow.cylc': None,
-                    'rincewind.txt': Path('whatever')
+                    'rincewind.txt': Symlink('whatever')
                 }}},
                 'sym': {'cylc-run': {'foo': {'bar': {}}}}
             }
@@ -547,12 +555,12 @@ def filetree_for_testing_cylc_clean(tmp_path: Path):
                 'cylc-run': {'foo': {'bar': {
                     '.service': {'db': None},
                     'flow.cylc': None,
-                    'log': Path('whatever'),
-                    'mirkwood': Path('whatever')
+                    'log': Symlink('whatever'),
+                    'mirkwood': Symlink('whatever')
                 }}},
                 'sym': {'cylc-run': {'foo': {'bar': {
                     'log': {
-                        'darmok': Path('whatever'),
+                        'darmok': Symlink('whatever'),
                         'bib': {}
                     }
                 }}}}
@@ -611,7 +619,7 @@ def test__clean_using_glob(
                 'cylc-run': {'foo': {'bar': {
                     '.service': {'db': None},
                     'flow.cylc': None,
-                    'rincewind.txt': Path('whatever')
+                    'rincewind.txt': Symlink('whatever')
                 }}},
                 'sym': {'cylc-run': {}}
             },
@@ -624,12 +632,12 @@ def test__clean_using_glob(
                 'cylc-run': {'foo': {'bar': {
                     '.service': {'db': None},
                     'flow.cylc': None,
-                    'log': Path('whatever'),
-                    'mirkwood': Path('whatever')
+                    'log': Symlink('whatever'),
+                    'mirkwood': Symlink('whatever')
                 }}},
                 'sym': {'cylc-run': {'foo': {'bar': {
                     'log': {
-                        'darmok': Path('whatever'),
+                        'darmok': Symlink('whatever'),
                         'bib': {}
                     }
                 }}}}
@@ -640,11 +648,13 @@ def test__clean_using_glob(
             {'**/cycle'},
             FILETREE_2,
             {
-                'cylc-run': {'foo': {'bar': Path('sym-run/cylc-run/foo/bar')}},
+                'cylc-run': {'foo': {
+                    'bar': Symlink('sym-run/cylc-run/foo/bar')
+                }},
                 'sym-run': {'cylc-run': {'foo': {'bar': {
                     '.service': {'db': None},
                     'flow.cylc': None,
-                    'share': Path('sym-share/cylc-run/foo/bar/share')
+                    'share': Symlink('sym-share/cylc-run/foo/bar/share')
                 }}}},
                 'sym-share': {'cylc-run': {'foo': {'bar': {
                     'share': {}
@@ -657,19 +667,15 @@ def test__clean_using_glob(
             {'share'},
             FILETREE_2,
             {
-                'cylc-run': {'foo': {'bar': Path('sym-run/cylc-run/foo/bar')}},
+                'cylc-run': {'foo': {
+                    'bar': Symlink('sym-run/cylc-run/foo/bar')
+                }},
                 'sym-run': {'cylc-run': {'foo': {'bar': {
                     '.service': {'db': None},
                     'flow.cylc': None,
                 }}}},
                 'sym-share': {'cylc-run': {}},
-                'sym-cycle': {'cylc-run': {'foo': {'bar': {
-                    'share': {
-                        'cycle': {
-                            'macklunkey.txt': None
-                        }
-                    }
-                }}}}
+                'sym-cycle': {'cylc-run': {}},
             },
             id="filetree2 share"
         ),
@@ -688,18 +694,14 @@ def test__clean_using_glob(
             {'*'},
             FILETREE_2,
             {
-                'cylc-run': {'foo': {'bar': Path('sym-run/cylc-run/foo/bar')}},
+                'cylc-run': {'foo': {
+                    'bar': Symlink('sym-run/cylc-run/foo/bar')
+                }},
                 'sym-run': {'cylc-run': {'foo': {'bar': {
                     '.service': {'db': None},
                 }}}},
                 'sym-share': {'cylc-run': {}},
-                'sym-cycle': {'cylc-run': {'foo': {'bar': {
-                    'share': {
-                        'cycle': {
-                            'macklunkey.txt': None
-                        }
-                    }
-                }}}}
+                'sym-cycle': {'cylc-run': {}},
             },
             id="filetree2 *"
         ),
@@ -752,7 +754,6 @@ def test_clean__targeted(
     """
     # --- Setup ---
     caplog.set_level(logging.DEBUG, CYLC_LOG)
-    tmp_run_dir()
     id_ = 'foo/bar'
     run_dir: Path
     files_to_delete: List[str]
@@ -780,7 +781,7 @@ def test_clean__targeted(
 def test_init_clean__targeted_bad(
     rm_dirs: List[str],
     tmp_run_dir: Callable,
-    monkeymock: MonkeyMock
+    monkeymock: 'MonkeyMock'
 ):
     """Test init_clean() fails when abusing --rm option."""
     tmp_run_dir('chalmers')
@@ -896,7 +897,7 @@ def test_remote_clean(
     expected_platforms: Optional[List[str]],
     exc_expected: bool,
     expected_err_msgs: List[str],
-    monkeymock: MonkeyMock, monkeypatch: pytest.MonkeyPatch,
+    monkeymock: 'MonkeyMock', monkeypatch: pytest.MonkeyPatch,
     caplog: pytest.LogCaptureFixture, log_filter: Callable
 ) -> None:
     """Test remote_clean() logic.
@@ -920,7 +921,7 @@ def test_remote_clean(
     # Remove randomness:
     monkeymock('cylc.flow.clean.shuffle')
 
-    def mocked_remote_clean_cmd_side_effect(id_, platform, rm_dirs, timeout):
+    def mocked_remote_clean_cmd_side_effect(id_, platform, timeout, rm_dirs):
         proc_ret_code = 0
         if failed_platforms and platform['name'] in failed_platforms:
             proc_ret_code = failed_platforms[platform['name']]
@@ -942,13 +943,15 @@ def test_remote_clean(
     if exc_expected:
         with pytest.raises(CylcError) as exc:
             cylc_clean.remote_clean(
-                id_, platform_names, rm_dirs, timeout='irrelevant')
+                id_, platform_names, timeout='irrelevant', rm_dirs=rm_dirs
+            )
         assert "Remote clean failed" in str(exc.value)
     else:
         cylc_clean.remote_clean(
-            id_, platform_names, rm_dirs, timeout='irrelevant')
+            id_, platform_names, timeout='irrelevant', rm_dirs=rm_dirs
+        )
     for msg in expected_err_msgs:
-        assert log_filter(caplog, level=logging.ERROR, contains=msg)
+        assert log_filter(logging.ERROR, msg)
     if expected_platforms:
         for p_name in expected_platforms:
             mocked_remote_clean_cmd.assert_any_call(
@@ -960,6 +963,36 @@ def test_remote_clean(
             assert f"{p_name} - {PlatformError.MSG_TIDY}" in caplog.text
 
 
+def test_remote_clean__timeout(
+    monkeymock: 'MonkeyMock',
+    monkeypatch: pytest.MonkeyPatch,
+    caplog: pytest.LogCaptureFixture,
+):
+    """Test remote_clean() gives a sensible error message for return code 124.
+    """
+    caplog.set_level(logging.ERROR, CYLC_LOG)
+    monkeymock(
+        'cylc.flow.clean._remote_clean_cmd',
+        spec=_remote_clean_cmd,
+        return_value=mock.Mock(
+            spec=Popen, poll=lambda: 124, communicate=lambda: ('', '')
+        )
+    )
+    monkeypatch.setattr(
+        'cylc.flow.clean.get_install_target_to_platforms_map',
+        lambda *a, **k: {'picard': [PLATFORMS['stargazer']]}
+    )
+
+    with pytest.raises(CylcError):
+        cylc_clean.remote_clean(
+            'blah', platform_names=['blah'], timeout='blah'
+        )
+    assert "cylc clean timed out" in caplog.text
+    # No need to log the remote clean cmd etc. for timeout
+    assert "ssh" not in caplog.text.lower()
+    assert "stderr" not in caplog.text.lower()
+
+
 @pytest.mark.parametrize(
     'rm_dirs, expected_args',
     [
@@ -971,7 +1004,7 @@ def test_remote_clean(
 def test_remote_clean_cmd(
     rm_dirs: Optional[List[str]],
     expected_args: List[str],
-    monkeymock: MonkeyMock
+    monkeymock: 'MonkeyMock'
 ) -> None:
     """Test _remote_clean_cmd()
 
@@ -1056,6 +1089,12 @@ def test_clean_top_level(tmp_run_dir: Callable):
              'cylc-run/foo/bar/share/cycle'],
             id="filetree2 **"
         ),
+        pytest.param(
+            'share',
+            FILETREE_2,
+            ['cylc-run/foo/bar/share'],
+            id="filetree2 share"
+        ),
         pytest.param(
             '**',
             FILETREE_3,
