@@ -14,75 +14,70 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import os
-from optparse import Values
-from typing import Any, Callable, Dict, List, Optional, Tuple, Type
-from pathlib import Path
-import pytest
-import logging
-from types import SimpleNamespace
-from unittest.mock import Mock
 from contextlib import suppress
+import logging
+from optparse import Values
+import os
+from pathlib import Path
+from textwrap import dedent
+from types import SimpleNamespace
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    List,
+    Optional,
+    Tuple,
+    Type,
+)
 
-from cylc.flow import CYLC_LOG
+import pytest
+
+from cylc.flow import (
+    CYLC_LOG,
+    flags,
+)
 from cylc.flow.config import WorkflowConfig
 from cylc.flow.cycling import loader
-from cylc.flow.cycling.loader import INTEGER_CYCLING_TYPE, ISO8601_CYCLING_TYPE
+from cylc.flow.cycling.iso8601 import ISO8601Point
+from cylc.flow.cycling.loader import (
+    INTEGER_CYCLING_TYPE,
+    ISO8601_CYCLING_TYPE,
+)
 from cylc.flow.exceptions import (
-    PointParsingError,
+    GraphParseError,
     InputError,
+    PointParsingError,
     WorkflowConfigError,
     XtriggerConfigError,
 )
+from cylc.flow.parsec.exceptions import (
+    Jinja2Error,
+)
 from cylc.flow.scheduler_cli import RunOptions
 from cylc.flow.scripts.validate import ValidateOptions
-from cylc.flow.workflow_files import WorkflowFiles
-from cylc.flow.wallclock import get_utc_mode, set_utc_mode
-from cylc.flow.xtrigger_mgr import XtriggerManager
 from cylc.flow.task_outputs import (
     TASK_OUTPUT_SUBMITTED,
-    TASK_OUTPUT_SUCCEEDED
+    TASK_OUTPUT_SUCCEEDED,
 )
-
-from cylc.flow.cycling.iso8601 import ISO8601Point
+from cylc.flow.wallclock import (
+    get_utc_mode,
+    set_utc_mode,
+)
+from cylc.flow.workflow_files import WorkflowFiles
 
 
 Fixture = Any
 
 
-def _tmp_flow_config(tmp_run_dir: Callable):
-    """Create a temporary flow config file for use in init'ing WorkflowConfig.
-
-    Args:
-        id_: Workflow name.
-        config: The flow file content.
-
-    Returns the path to the flow file.
-    """
-    def __tmp_flow_config(id_: str, config: str) -> Path:
-        run_dir: Path = tmp_run_dir(id_)
-        flow_file = run_dir / WorkflowFiles.FLOW_FILE
-        flow_file.write_text(config)
-        return flow_file
-    return __tmp_flow_config
-
-
-@pytest.fixture
-def tmp_flow_config(tmp_run_dir: Callable):
-    return _tmp_flow_config(tmp_run_dir)
-
-
-@pytest.fixture(scope='module')
-def mod_tmp_flow_config(mod_tmp_run_dir: Callable):
-    return _tmp_flow_config(mod_tmp_run_dir)
+param = pytest.param
 
 
 class TestWorkflowConfig:
     """Test class for the Cylc WorkflowConfig object."""
 
     def test_xfunction_imports(
-            self, mock_glbl_cfg: Fixture, tmp_path: Path,
-            xtrigger_mgr: XtriggerManager):
+            self, mock_glbl_cfg: 'Fixture', tmp_path: 'Path'):
         """Test for a workflow configuration with valid xtriggers"""
         mock_glbl_cfg(
             'cylc.flow.platforms.glbl_cfg',
@@ -110,10 +105,9 @@ class TestWorkflowConfig:
         """
         flow_file.write_text(flow_config)
         workflow_config = WorkflowConfig(
-            workflow="name_a_tree", fpath=flow_file, options=Mock(spec=[]),
-            xtrigger_mgr=xtrigger_mgr
+            workflow="name_a_tree", fpath=flow_file, options=SimpleNamespace()
         )
-        assert 'tree' in workflow_config.xtrigger_mgr.functx_map
+        assert 'tree' in workflow_config.xtrigger_collator.functx_map
 
     def test_xfunction_import_error(self, mock_glbl_cfg, tmp_path):
         """Test for error when a xtrigger function cannot be imported."""
@@ -144,9 +138,9 @@ class TestWorkflowConfig:
             WorkflowConfig(
                 workflow="caiman_workflow",
                 fpath=flow_file,
-                options=Mock(spec=[])
+                options=SimpleNamespace()
             )
-        assert "not found" in str(excinfo.value)
+        assert "No module named 'piranha'" in str(excinfo.value)
 
     def test_xfunction_attribute_error(self, mock_glbl_cfg, tmp_path):
         """Test for error when a xtrigger function cannot be imported."""
@@ -175,8 +169,9 @@ class TestWorkflowConfig:
         flow_file.write_text(flow_config)
         with pytest.raises(XtriggerConfigError) as excinfo:
             WorkflowConfig(workflow="capybara_workflow", fpath=flow_file,
-                           options=Mock(spec=[]))
-        assert "not found" in str(excinfo.value)
+                           options=SimpleNamespace())
+        assert "module 'capybara' has no attribute 'capybara'" in str(
+            excinfo.value)
 
     def test_xfunction_not_callable(self, mock_glbl_cfg, tmp_path):
         """Test for error when a xtrigger function is not callable."""
@@ -207,7 +202,7 @@ class TestWorkflowConfig:
             WorkflowConfig(
                 workflow="workflow_with_not_callable",
                 fpath=flow_file,
-                options=Mock(spec=[])
+                options=SimpleNamespace()
             )
         assert "callable" in str(excinfo.value)
 
@@ -268,7 +263,7 @@ def test_family_inheritance_and_quotes(
 
 
 @pytest.mark.parametrize(
-    ('cycling_type', 'scheduling_cfg', 'expected_icp', 'expected_opt_icp',
+    ('cycling_type', 'scheduling_cfg', 'expected_icp', 'expected_eval_icp',
      'expected_err'),
     [
         pytest.param(
@@ -357,9 +352,9 @@ def test_process_icp(
     cycling_type: str,
     scheduling_cfg: Dict[str, Any],
     expected_icp: Optional[str],
-    expected_opt_icp: Optional[str],
+    expected_eval_icp: Optional[str],
     expected_err: Optional[Tuple[Type[Exception], str]],
-    monkeypatch: pytest.MonkeyPatch, set_cycling_type: Fixture
+    monkeypatch: pytest.MonkeyPatch, set_cycling_type: 'Fixture'
 ) -> None:
     """Test WorkflowConfig.process_initial_cycle_point().
 
@@ -369,19 +364,21 @@ def test_process_icp(
         cycling_type: Workflow cycling type.
         scheduling_cfg: 'scheduling' section of workflow config.
         expected_icp: The expected icp value that gets set.
-        expected_opt_icp: The expected value of options.icp that gets set
+        expected_eval_icp: The expected value of options.icp that gets set
             (this gets stored in the workflow DB).
         expected_err: Exception class expected to be raised plus the message.
     """
     set_cycling_type(cycling_type, time_zone="+0530")
-    mocked_config = Mock(cycling_type=cycling_type)
-    mocked_config.cfg = {
-        'scheduling': {
-            'initial cycle point constraints': [],
-            **scheduling_cfg
-        }
-    }
-    mocked_config.options.icp = None
+    mocked_config = SimpleNamespace(
+        cycling_type=cycling_type,
+        options=SimpleNamespace(icp=None),
+        cfg={
+            'scheduling': {
+                'initial cycle point constraints': [],
+                **scheduling_cfg
+            },
+        },
+    )
     monkeypatch.setattr('cylc.flow.config.get_current_time_string',
                         lambda: '20050102T0615+0530')
 
@@ -395,10 +392,10 @@ def test_process_icp(
         assert mocked_config.cfg[
             'scheduling']['initial cycle point'] == expected_icp
         assert str(mocked_config.initial_point) == expected_icp
-        opt_icp = mocked_config.options.icp
-        if opt_icp is not None:
-            opt_icp = str(loader.get_point(opt_icp).standardise())
-        assert opt_icp == expected_opt_icp
+        eval_icp = mocked_config.evaluated_icp
+        if eval_icp is not None:
+            eval_icp = str(loader.get_point(eval_icp).standardise())
+        assert eval_icp == expected_eval_icp
 
 
 @pytest.mark.parametrize(
@@ -444,7 +441,7 @@ def test_process_startcp(
     starttask: Optional[str],
     expected: str,
     expected_err: Optional[Tuple[Type[Exception], str]],
-    monkeypatch: pytest.MonkeyPatch, set_cycling_type: Fixture
+    monkeypatch: pytest.MonkeyPatch, set_cycling_type: 'Fixture'
 ) -> None:
     """Test WorkflowConfig.process_start_cycle_point().
 
@@ -457,9 +454,10 @@ def test_process_startcp(
         expected_err: Expected exception.
     """
     set_cycling_type(ISO8601_CYCLING_TYPE, time_zone="+0530")
-    mocked_config = Mock(initial_point='18990501T0000+0530')
-    mocked_config.options.startcp = startcp
-    mocked_config.options.starttask = starttask
+    mocked_config = SimpleNamespace(
+        initial_point='18990501T0000+0530',
+        options=SimpleNamespace(startcp=startcp, starttask=starttask),
+    )
     monkeypatch.setattr('cylc.flow.config.get_current_time_string',
                         lambda: '20050102T0615+0530')
     if expected_err is not None:
@@ -532,6 +530,17 @@ def test_process_startcp(
             None,
             id="Relative fcp"
         ),
+        pytest.param(
+            ISO8601_CYCLING_TYPE,
+            {
+                'initial cycle point': '2017-02-11',
+                'final cycle point': '+P4D+PT3H-PT2H',
+            },
+            None,
+            '20170215T0100+0530',
+            None,
+            id="Relative fcp chained"
+        ),
         pytest.param(
             ISO8601_CYCLING_TYPE,
             {
@@ -635,7 +644,7 @@ def test_process_fcp(
     options_fcp: Optional[str],
     expected_fcp: Optional[str],
     expected_err: Optional[Tuple[Type[Exception], str]],
-    set_cycling_type: Fixture
+    set_cycling_type: 'Fixture'
 ) -> None:
     """Test WorkflowConfig.process_final_cycle_point().
 
@@ -647,17 +656,20 @@ def test_process_fcp(
         expected_err: Exception class expected to be raised plus the message.
     """
     set_cycling_type(cycling_type, time_zone='+0530')
-    mocked_config = Mock(cycling_type=cycling_type)
-    mocked_config.cfg = {
-        'scheduling': {
-            'final cycle point constraints': [],
-            **scheduling_cfg
-        }
-    }
-    mocked_config.initial_point = loader.get_point(
-        scheduling_cfg['initial cycle point']).standardise()
-    mocked_config.final_point = None
-    mocked_config.options.fcp = options_fcp
+    mocked_config = SimpleNamespace(
+        cycling_type=cycling_type,
+        cfg={
+            'scheduling': {
+                'final cycle point constraints': [],
+                **scheduling_cfg,
+            },
+        },
+        initial_point=loader.get_point(
+            scheduling_cfg['initial cycle point']
+        ).standardise(),
+        final_point=None,
+        options=SimpleNamespace(fcp=options_fcp),
+    )
 
     if expected_err:
         err, msg = expected_err
@@ -677,24 +689,28 @@ def test_process_fcp(
     [
         pytest.param(
             None, None, None, None, None,
-            id="No stopcp"
+            id="no-stopcp"
         ),
         pytest.param(
             '1993', None, '1993', None, None,
-            id="From config by default"
+            id="stopcp"
         ),
         pytest.param(
             '1993', '1066', '1066', '1066', None,
-            id="From options"
+            id="stop-cp-and-cli-option"
         ),
         pytest.param(
             '1993', 'reload', '1993', None, None,
-            id="From cfg if --stopcp=reload on restart"
+            id="stop-cp-and-cli-reload-option"
         ),
         pytest.param(
             '3000', None, None, None,
             "will have no effect as it is after the final cycle point",
-            id="stopcp > fcp"
+            id="stopcp-beyond-fcp"
+        ),
+        pytest.param(
+            '+P12Y -P2Y', None, '2000', None, None,
+            id="stopcp-relative-to-icp"
         ),
     ]
 )
@@ -719,12 +735,13 @@ def test_process_stop_cycle_point(
     set_cycling_type(ISO8601_CYCLING_TYPE, dump_format='CCYY')
     caplog.set_level(logging.WARNING, CYLC_LOG)
     fcp = loader.get_point('2012').standardise()
-    mock_config = Mock(
+    mock_config = SimpleNamespace(
         cfg={
             'scheduling': {
                 'stop after cycle point': cfg_stopcp
             }
         },
+        initial_point=ISO8601Point('1990'),
         final_point=fcp,
         stop_point=None,
         options=RunOptions(stopcp=options_stopcp),
@@ -791,7 +808,7 @@ def test_stopcp_after_fcp(
     cycle point is handled correctly."""
     caplog.set_level(logging.WARNING, CYLC_LOG)
     id_ = 'cassini'
-    flow_file: Path = tmp_flow_config(id_, f"""
+    flow_file: 'Path' = tmp_flow_config(id_, f"""
     [scheduler]
         allow implicit tasks = True
     [scheduling]
@@ -872,7 +889,7 @@ def test_prelim_process_graph(
             processing.
         expected_err: Exception class expected to be raised plus the message.
     """
-    mock_config = Mock(cfg={
+    mock_config = SimpleNamespace(cfg={
         'scheduling': scheduling_cfg
     })
 
@@ -898,13 +915,15 @@ def test_utc_mode(caplog, mock_glbl_cfg):
                 UTC mode = {utc_mode['glbl']}
             '''
         )
-        mock_config = Mock()
-        mock_config.cfg = {
-            'scheduler': {
-                'UTC mode': utc_mode['workflow']
-            }
-        }
-        mock_config.options.utc_mode = utc_mode['stored']
+        mock_config = SimpleNamespace(
+            cfg={
+                'scheduler': {
+                    'UTC mode': utc_mode['workflow']
+                }
+            },
+            options=SimpleNamespace(utc_mode=utc_mode['stored']),
+        )
+
         WorkflowConfig.process_utc_mode(mock_config)
         assert mock_config.cfg['scheduler']['UTC mode'] is expected
         assert get_utc_mode() is expected
@@ -948,13 +967,15 @@ def test_cycle_point_tz(caplog, monkeypatch):
 
     def _test(cp_tz, utc_mode, expected, expected_warnings=0):
         set_utc_mode(utc_mode)
-        mock_config = Mock()
-        mock_config.cfg = {
-            'scheduler': {
-                'cycle point time zone': cp_tz['workflow']
-            }
-        }
-        mock_config.options.cycle_point_tz = cp_tz['stored']
+        mock_config = SimpleNamespace(
+            cfg={
+                'scheduler': {
+                    'cycle point time zone': cp_tz['workflow'],
+                },
+            },
+            options=SimpleNamespace(cycle_point_tz=cp_tz['stored']),
+        )
+
         WorkflowConfig.process_cycle_point_tz(mock_config)
         assert mock_config.cfg['scheduler'][
             'cycle point time zone'] == expected
@@ -1019,6 +1040,36 @@ def test_rsync_includes_will_not_accept_sub_directories(tmp_flow_config):
     assert "Directories can only be from the top level" in str(exc.value)
 
 
+@pytest.mark.parametrize(
+    'cylc_var, expected_err',
+    [
+        ["CYLC_WORKFLOW_NAME", None],
+        ["CYLC_BEEF_WELLINGTON", (Jinja2Error, "is undefined")],
+    ]
+)
+def test_jinja2_cylc_vars(tmp_flow_config, cylc_var, expected_err):
+    """Defined CYLC_ variables should be available to Jinja2 during parsing.
+
+    This test is not located in the jinja2_support unit test module because
+    CYLC_ variables are only defined during workflow config parsing.
+    """
+    reg = 'nodule'
+    flow_file = tmp_flow_config(reg, """#!Jinja2
+    # {{""" + cylc_var + """}}
+    [scheduler]
+        allow implicit tasks = True
+    [scheduling]
+        [[graph]]
+            R1 = foo
+    """)
+    if expected_err is None:
+        WorkflowConfig(workflow=reg, fpath=flow_file, options=Values())
+    else:
+        with pytest.raises(expected_err[0]) as exc:
+            WorkflowConfig(workflow=reg, fpath=flow_file, options=Values())
+        assert expected_err[1] in str(exc)
+
+
 def test_valid_rsync_includes_returns_correct_list(tmp_flow_config):
     """Test that the rsync includes in the correct """
     id_ = 'rsynctest'
@@ -1062,7 +1113,7 @@ def test_process_runahead_limit(
     set_cycling_type: Callable
 ) -> None:
     set_cycling_type(cycling_type)
-    mock_config = Mock(cycling_type=cycling_type)
+    mock_config = SimpleNamespace(cycling_type=cycling_type)
     mock_config.cfg = {
         'scheduling': {
             'runahead limit': runahead_limit
@@ -1082,9 +1133,9 @@ def test_process_runahead_limit(
 def test_check_circular(opt, monkeypatch, caplog, tmp_flow_config):
     """Test WorkflowConfig._check_circular()."""
     # ----- Setup -----
-    caplog.set_level(logging.WARNING, CYLC_LOG)
+    caplog.set_level(logging.INFO, CYLC_LOG)
 
-    options = Mock(spec=[], is_validate=True)
+    options = SimpleNamespace(is_validate=True)
     if opt:
         setattr(options, opt, True)
 
@@ -1118,13 +1169,16 @@ def test_check_circular(opt, monkeypatch, caplog, tmp_flow_config):
         WorkflowConfig__assert_err_raised()
 
 
-def test_undefined_custom_output(tmp_flow_config: Callable):
+@pytest.mark.parametrize(
+    'graph', (('foo:x => bar'), ('foo:x'))
+)
+def test_undefined_custom_output(graph: str, tmp_flow_config: Callable):
     """Test error on undefined custom output referenced in graph."""
     id_ = 'custom_out1'
-    flow_file = tmp_flow_config(id_, """
+    flow_file = tmp_flow_config(id_, f"""
     [scheduling]
         [[graph]]
-            R1 = "foo:x => bar"
+            R1 = "{graph}"
     [runtime]
         [[foo, bar]]
     """)
@@ -1216,28 +1270,6 @@ def test_implicit_success_required(tmp_flow_config, graph):
     assert cfg.taskdefs['foo'].outputs[TASK_OUTPUT_SUCCEEDED][1]
 
 
-@pytest.mark.parametrize(
-    'graph',
-    [
-        "foo:submit? => bar",
-        "foo:submit-fail? => bar",
-    ]
-)
-def test_success_after_optional_submit(tmp_flow_config, graph):
-    """Check foo:succeed is not required if foo:submit is optional."""
-    id_ = 'blargh'
-    flow_file = tmp_flow_config(id_, f"""
-    [scheduling]
-        [[graph]]
-            R1 = {graph}
-    [runtime]
-        [[bar]]
-        [[foo]]
-    """)
-    cfg = WorkflowConfig(workflow=id_, fpath=flow_file, options=None)
-    assert not cfg.taskdefs['foo'].outputs[TASK_OUTPUT_SUCCEEDED][1]
-
-
 @pytest.mark.parametrize(
     'allow_implicit_tasks',
     [
@@ -1292,16 +1324,21 @@ def test_implicit_tasks(
     """
     # Setup
     id_ = 'rincewind'
-    flow_file: Path = tmp_flow_config(id_, f"""
-    [scheduler]
-        {
-            f'allow implicit tasks = {allow_implicit_tasks}'
-            if allow_implicit_tasks is not None else ''
-        }
-    [scheduling]
-        [[graph]]
-            R1 = foo
-    """)
+
+    allow_implicit_tasks_text = (
+        f'allow implicit tasks = {allow_implicit_tasks}'
+        if allow_implicit_tasks is not None else ''
+    )
+    flow_file: 'Path' = tmp_flow_config(
+        id_,
+        dedent(f"""
+            [scheduler]
+                {allow_implicit_tasks_text}
+            [scheduling]
+                [[graph]]
+                    R1 = foo
+        """)
+    )
     monkeypatch.setattr('cylc.flow.flags.cylc7_back_compat', cylc7_compat)
     if rose_suite_conf:
         (flow_file.parent / 'rose-suite.conf').touch()
@@ -1326,7 +1363,7 @@ def test_implicit_tasks(
 
 @pytest.mark.parametrize('workflow_meta', [True, False])
 @pytest.mark.parametrize('url_type', ['good', 'bad', 'ugly', 'broken'])
-def test_process_urls(caplog, log_filter, workflow_meta, url_type):
+def test_process_urls(log_filter, workflow_meta, url_type):
 
     if url_type == 'good':
         # valid cylc 8 syntax
@@ -1362,7 +1399,6 @@ def test_process_urls(caplog, log_filter, workflow_meta, url_type):
     elif url_type == 'ugly':
         WorkflowConfig.process_metadata_urls(config)
         assert log_filter(
-            caplog,
             contains='Detected deprecated template variables',
         )
 
@@ -1390,13 +1426,12 @@ def test_zero_interval(
     should_warn: bool,
     opts: Values,
     tmp_flow_config: Callable,
-    caplog: pytest.LogCaptureFixture,
     log_filter: Callable,
 ):
     """Test that a zero-duration recurrence with >1 repetition gets an
     appropriate warning."""
     id_ = 'ordinary'
-    flow_file: Path = tmp_flow_config(id_, f"""
+    flow_file: 'Path' = tmp_flow_config(id_, f"""
     [scheduler]
         UTC mode = True
         allow implicit tasks = True
@@ -1408,7 +1443,6 @@ def test_zero_interval(
     """)
     WorkflowConfig(id_, flow_file, options=opts)
     logged = log_filter(
-        caplog,
         level=logging.WARNING,
         contains="Cannot have more than 1 repetition for zero-duration"
     )
@@ -1429,11 +1463,7 @@ def test_zero_interval(
         ('1988-02-29', '+P1M+P1Y', '1989-03-29'),
         ('1910-08-14', '+P2D-PT6H', '1910-08-15T18:00'),
         ('1850-04-10', '+P1M-P1D+PT1H', '1850-05-09T01:00'),
-        pytest.param(
-            '1066-10-14', '+PT1H+PT1M', '1066-10-14T01:01',
-            marks=pytest.mark.xfail
-            # https://github.com/cylc/cylc-flow/issues/5047
-        ),
+        ('1066-10-14', '+PT1H+PT1M', '1066-10-14T01:01'),
     ]
 )
 def test_chain_expr(
@@ -1444,7 +1474,7 @@ def test_chain_expr(
     Note the order matters when "nominal" units (years, months) are used.
     """
     id_ = 'osgiliath'
-    flow_file: Path = tmp_flow_config(id_, f"""
+    flow_file: 'Path' = tmp_flow_config(id_, f"""
         [scheduler]
             UTC mode = True
             allow implicit tasks = True
@@ -1623,7 +1653,7 @@ def test__warn_if_queues_have_implicit_tasks(caplog):
     ]
 )
 def test_cylc_env_at_parsing(
-    tmp_path: Path,
+    tmp_path: 'Path',
     monkeypatch: pytest.MonkeyPatch,
     installed,
     run_dir,
@@ -1657,7 +1687,7 @@ def test_cylc_env_at_parsing(
 
     # Parse the workflow config then check the environment.
     WorkflowConfig(
-        workflow="name", fpath=flow_file, options=Mock(spec=[]),
+        workflow="name", fpath=flow_file, options=SimpleNamespace(),
         run_dir=run_dir
     )
 
@@ -1668,3 +1698,116 @@ def test_cylc_env_at_parsing(
             assert var in cylc_env
         else:
             assert var not in cylc_env
+
+
+def test_force_workflow_compat_mode(tmp_path):
+    fpath = (tmp_path / 'flow.cylc')
+    fpath.write_text(dedent("""
+        [scheduler]
+            allow implicit tasks = true
+        [scheduling]
+            [[graph]]
+                R1 = a:succeeded | a:failed => b
+    """))
+    # It fails without compat mode:
+    with pytest.raises(GraphParseError, match='Opposite outputs'):
+        WorkflowConfig('foo', str(fpath), {})
+    # It succeeds with compat mode:
+    WorkflowConfig('foo', str(fpath), {}, force_compat_mode=True)
+
+
+@pytest.mark.parametrize(
+    'registered_outputs, tasks_and_outputs, fails',
+    (
+        param([], ['foo:x'], True, id='output-unregistered'),
+        param([], ['foo:x?'], True, id='optional-output-unregistered'),
+        param([], ['foo'], False, id='no-modifier-unregistered'),
+        param(['x'], ['foo:x'], False, id='output-registered'),
+        param([], ['foo:succeed'], False, id='alt-default-ok'),
+        param([], ['foo:failed'], False, id='default-ok'),
+    )
+)
+def test_check_outputs(tmp_path, registered_outputs, tasks_and_outputs, fails):
+    (tmp_path / 'flow.cylc').write_text(dedent("""
+        [scheduler]
+            allow implicit tasks = true
+        [scheduling]
+            [[graph]]
+                R1 = foo
+    """))
+    cfg = WorkflowConfig('', tmp_path / 'flow.cylc', '')
+    cfg.cfg['runtime']['foo']['outputs'] = registered_outputs
+    if fails:
+        with pytest.raises(
+            WorkflowConfigError, match='Undefined custom output'
+        ):
+            cfg.check_terminal_outputs(tasks_and_outputs)
+    else:
+        assert cfg.check_terminal_outputs(tasks_and_outputs) is None
+
+
+@pytest.mark.parametrize('back_compat', [True, False])
+def test_upg_wflow_event_names(back_compat, tmp_flow_config, log_filter):
+    """Cylc 7 workflow handler/mail event names are upgraded."""
+    flags.cylc7_back_compat = back_compat
+    events = 'inactivity, abort, stalled'
+    expected = ['inactivity timeout', 'abort', 'stall']
+    flow_file = tmp_flow_config('foo', f"""
+        [scheduler]
+            allow implicit tasks = true
+            [[events]]
+                handler events = {events}
+                mail events = {events}
+        [scheduling]
+            [[graph]]
+                R1 = foo
+    """)
+    cfg = WorkflowConfig('foo', str(flow_file), ValidateOptions())
+    for item in ('handler events', 'mail events'):
+        assert cfg.cfg['scheduler']['events'][item] == expected
+    if back_compat:
+        assert not log_filter(logging.WARNING)
+    else:
+        assert log_filter(
+            logging.WARNING,
+            'Deprecated config items were automatically upgraded',
+        )
+
+
+@pytest.mark.parametrize('item', ['handler events', 'mail events'])
+def test_val_wflow_event_names(item, tmp_flow_config, log_filter):
+    """Any invalid workflow handler/mail events raise a warning."""
+    flow_file = tmp_flow_config('foo', f"""
+        [scheduler]
+            allow implicit tasks = true
+            [[events]]
+                {item} = abort, badger, stall, alpaca
+        [scheduling]
+            [[graph]]
+                R1 = foo
+    """)
+    WorkflowConfig('foo', str(flow_file), ValidateOptions())
+    assert log_filter(contains=f"{item} = badger, alpaca")
+
+
+@pytest.mark.parametrize('item', ['handler events', 'mail events'])
+def test_check_task_event_names(item, tmp_flow_config, log_filter):
+    """"Any invalid task handler events are warned about."""
+    flow_file = tmp_flow_config('foo', f"""
+        [scheduling]
+            [[graph]]
+                R1 = foo
+        [runtime]
+            [[foo]]
+                [[[events]]]
+                    {item} = submitted, late, warning, custom,\
+                        execution timeout, badger, owl, retry, horse
+                [[[outputs]]]
+                    owl = who
+    """)
+
+    WorkflowConfig('foo', str(flow_file), ValidateOptions())
+    assert log_filter(contains=(
+        f"Invalid event name(s) for [runtime][foo][events]{item}: "
+        "badger, horse"
+    ))
