@@ -34,6 +34,7 @@ from cylc.flow.exceptions import (
     PlatformLookupError,
     GlobalConfigError
 )
+from cylc.flow.run_modes import JOBLESS_MODES
 
 
 PLATFORMS = {
@@ -272,6 +273,16 @@ def test_platform_name_from_job_info_basic(job, remote, returns):
     assert platform_name_from_job_info(PLATFORMS, job, remote) == returns
 
 
+def test_platform_name_from_job_info_evaluated_hostname():
+    result = platform_name_from_job_info(
+        PLATFORMS,
+        {'batch system': 'background'},
+        {'host': '$(cat tiddles)'},
+        evaluated_host='hpc2',
+    )
+    assert result == 'hpc2-bg'
+
+
 def test_platform_name_from_job_info_ordered_dict_comparison():
     """Check that we are only comparing set items in OrderedDictWithDefaults.
     """
@@ -473,6 +484,21 @@ def test_get_install_target_to_platforms_map(
         assert result == expected_map
 
 
+@pytest.mark.parametrize('mode', sorted(JOBLESS_MODES))
+def test_platform_from_name__jobless_modes(mode):
+    result = platform_from_name(mode)
+    assert result['name'] == 'localhost'
+
+
+@pytest.mark.parametrize('mode', sorted(JOBLESS_MODES))
+def test_get_install_target_to_platforms_map__jobless_modes(mode):
+    result = get_install_target_to_platforms_map([mode])
+    assert list(result) == ['localhost']
+    assert len(result['localhost']) == 1
+    assert result['localhost'][0]['hosts'] == ['localhost']
+    assert result['localhost'][0]['install target'] == 'localhost'
+
+
 @pytest.mark.parametrize(
     'platform, job, remote, expect',
     [
