@@ -36,7 +36,7 @@ from cylc.flow.wallclock import (
 )
 
 
-def testrender_node__job_info():
+def test_render_node__job_info():
     """It renders job information nodes."""
     assert render_node(
         None,
@@ -48,7 +48,7 @@ def testrender_node__job_info():
     ]
 
 
-def testrender_node__job():
+def test_render_node__job():
     """It renders job nodes."""
     assert render_node(
         None,
@@ -60,7 +60,7 @@ def testrender_node__job():
     ]
 
 
-def testrender_node__task__succeeded():
+def test_render_node__task__succeeded():
     """It renders tasks."""
     node = Mock()
     node.get_child_node = lambda _: None
@@ -71,17 +71,18 @@ def testrender_node__task__succeeded():
             'state': 'succeeded',
             'isHeld': False,
             'isQueued': False,
-            'isRunahead': False
+            'isRunahead': False,
+            'flowNums': '[1]',
         },
         'task'
     ) == [
-        TASK_ICONS['succeeded'],
+        ('body', TASK_ICONS['succeeded']),
         ' ',
-        'foo'
+        ('body', 'foo'),
     ]
 
 
-def testrender_node__task__running():
+def test_render_node__task__running():
     """It renders running tasks."""
     child = Mock()
     child.get_value = lambda: {'data': {
@@ -98,19 +99,20 @@ def testrender_node__task__running():
             'isHeld': False,
             'isQueued': False,
             'isRunahead': False,
+            'flowNums': '[1]',
             'task': {'meanElapsedTime': 100}
         },
         'task'
     ) == [
-        TASK_ICONS['running'],
+        ('body', TASK_ICONS['running']),
         ' ',
         ('job_running', JOB_ICON),
         ' ',
-        'foo'
+        ('body', 'foo'),
     ]
 
 
-def testrender_node__family():
+def test_render_node__family():
     """It renders families."""
     assert render_node(
         None,
@@ -123,13 +125,13 @@ def testrender_node__family():
         },
         'family'
     ) == [
-        [TASK_ICONS['succeeded']],
+        [('body', TASK_ICONS['succeeded'])],
         ' ',
         'myid'
     ]
 
 
-def testrender_node__cycle_point():
+def test_render_node__cycle_point():
     """It renders cycle points."""
     assert render_node(
         None,
@@ -159,7 +161,7 @@ def testrender_node__cycle_point():
         # is-queued modifier
         ('waiting', False, True, False, None, None, ['\u033F', '○']),
         # is-runahead modifier
-        ('waiting', False, False, True, None, None, ['\u030A', '○'])
+        ('waiting', False, False, True, None, None, ['\u0340', '○']),
     ]
 )
 def test_get_task_icon(
@@ -172,12 +174,18 @@ def test_get_task_icon(
             datetime.utcnow() - timedelta(seconds=start_offset)
         )
     assert (
-        get_task_icon(
-            status, is_held=is_held, is_queued=is_queued,
-            is_runahead=is_runahead, start_time=start_time,
-            mean_time=mean_time
+        (
+            get_task_icon(
+                status,
+                is_held=is_held,
+                is_queued=is_queued,
+                is_runahead=is_runahead,
+                colour='custom',
+                start_time=start_time,
+                mean_time=mean_time,
+            )
         )
-    ) == expected
+    ) == [('custom', char) for char in expected]
 
 
 def test_compute_tree():
@@ -189,77 +197,87 @@ def test_compute_tree():
 
     """
     tree = compute_tree({
-        'id': 'workflow id',
-        'cyclePoints': [
-            {
-                'id': '1/family-suffix',
-                'cyclePoint': '1'
-            }
-        ],
-        'familyProxies': [
-            {  # top level family
-                'name': 'FOO',
-                'id': '1/FOO',
-                'cyclePoint': '1',
-                'firstParent': {'name': 'root', 'id': '1/root'}
-            },
-            {  # nested family
-                'name': 'FOOT',
-                'id': '1/FOOT',
-                'cyclePoint': '1',
-                'firstParent': {'name': 'FOO', 'id': '1/FOO'}
-            },
-        ],
-        'taskProxies': [
-            {  # top level task
-                'name': 'pub',
-                'id': '1/pub',
-                'firstParent': {'name': 'root', 'id': '1/root'},
-                'cyclePoint': '1',
-                'jobs': []
-            },
-            {  # child task (belongs to family)
-                'name': 'fan',
-                'id': '1/fan',
-                'firstParent': {'name': 'fan', 'id': '1/fan'},
-                'cyclePoint': '1',
-                'jobs': []
-            },
-            {  # nested child task (belongs to incestuous family)
-                'name': 'fool',
-                'id': '1/fool',
-                'firstParent': {'name': 'FOOT', 'id': '1/FOOT'},
-                'cyclePoint': '1',
-                'jobs': []
-            },
-            {  # a task which has jobs
-                'name': 'worker',
-                'id': '1/worker',
-                'firstParent': {'name': 'root', 'id': '1/root'},
-                'cyclePoint': '1',
-                'jobs': [
-                    {'id': '1/worker/03', 'submitNum': '3'},
-                    {'id': '1/worker/02', 'submitNum': '2'},
-                    {'id': '1/worker/01', 'submitNum': '1'}
-                ]
-            }
-        ]
+        'workflows': [{
+            'id': 'workflow id',
+            'port': 1234,
+            'cyclePoints': [
+                {
+                    'id': '1/family-suffix',
+                    'cyclePoint': '1'
+                }
+            ],
+            'familyProxies': [
+                {  # top level family
+                    'name': 'FOO',
+                    'id': '1/FOO',
+                    'cyclePoint': '1',
+                    'firstParent': {'name': 'root', 'id': '1/root'}
+                },
+                {  # nested family
+                    'name': 'FOOT',
+                    'id': '1/FOOT',
+                    'cyclePoint': '1',
+                    'firstParent': {'name': 'FOO', 'id': '1/FOO'}
+                },
+            ],
+            'taskProxies': [
+                {  # top level task
+                    'name': 'pub',
+                    'id': '1/pub',
+                    'firstParent': {'name': 'root', 'id': '1/root'},
+                    'cyclePoint': '1',
+                    'jobs': []
+                },
+                {  # child task (belongs to family)
+                    'name': 'fan',
+                    'id': '1/fan',
+                    'firstParent': {'name': 'fan', 'id': '1/fan'},
+                    'cyclePoint': '1',
+                    'jobs': []
+                },
+                {  # nested child task (belongs to incestuous family)
+                    'name': 'fool',
+                    'id': '1/fool',
+                    'firstParent': {'name': 'FOOT', 'id': '1/FOOT'},
+                    'cyclePoint': '1',
+                    'jobs': []
+                },
+                {  # a task which has jobs
+                    'name': 'worker',
+                    'id': '1/worker',
+                    'firstParent': {'name': 'root', 'id': '1/root'},
+                    'cyclePoint': '1',
+                    'jobs': [
+                        {'id': '1/worker/03', 'submitNum': '3'},
+                        {'id': '1/worker/02', 'submitNum': '2'},
+                        {'id': '1/worker/01', 'submitNum': '1'}
+                    ]
+                }
+            ]
+        }]
     })
 
-    # the workflow node
-    assert tree['type_'] == 'workflow'
-    assert tree['id_'] == 'workflow id'
-    assert list(tree['data']) == [
-        # whatever if present on the node should end up in data
-        'id',
-        'cyclePoints',
-        'familyProxies',
-        'taskProxies'
-    ]
+    # the root node
+    assert tree['type_'] == 'root'
+    assert tree['id_'] == 'root'
     assert len(tree['children']) == 1
 
+    # the workflow node
+    workflow = tree['children'][0]
+    assert workflow['type_'] == 'workflow'
+    assert workflow['id_'] == 'workflow id'
+    assert set(workflow['data']) == {
+        # whatever if present on the node should end up in data
+        'cyclePoints',
+        'familyProxies',
+        'id',
+        'port',
+        'taskProxies'
+    }
+    assert len(workflow['children']) == 1
+
     # the cycle point node
-    cycle = tree['children'][0]
+    cycle = workflow['children'][0]
     assert cycle['type_'] == 'cycle'
     assert cycle['id_'] == '//1'
     assert list(cycle['data']) == [
